from __future__ import print_function
import Manyo as mm
import utsusemi.ana.Reduction.BaseCommandsInEla as BaseCom
import Manyo.LevmarTools as ml
import Manyo.Utsusemi as mu

import uGao.MPlot as mp
import os,sys,time,math

ver = float(sys.version_info[0])+float(sys.version_info[1])*0.1
if ver<2.5:
    from elementtree.ElementTree import ElementTree
else:
    from xml.etree.ElementTree import ElementTree

import utsusemi.DNA.ana.Reduction.SearchPathDNA          as SPD

convList = ["lc","gc","dc","pv1c","pv2c","dhoc","alc","cc","p1c","p2c","p3c"]
nullEC=mm.ElementContainer()
nullEC.Add("x",[x*0.01 for x in range(101)])
nullEC.Add("y",[1]*100)
nullEC.Add("e",[1]*100)
nullEC.SetKeys("x","y","e")

#############################################################################
#
#def ReadDataEC( filename ):
#    """
#    @param filename   (string)
#    @retval ec (ElementConteiner)
#    """
#    ec = mm.ElementContainer()
#    flug = os.path.exists(filename)
#    if flug:
#        Min = mm.ReadSerializationFileBinary( filename )
#        Min.Load( ec )
#        del(Min)
#    else:
#        print "ReadData >> ",filename,"is not exist"
#    return ec
#
#############################################################################
#
#def ReadDataECA( filename ):
#    """
#    @param filename   (string)
#    @retval eca (ElementConteinerArray)
#    """
#    eca = mm.ElementContainerArray()
#    flug = os.path.exists(filename)
#    if flug:
#        Min = mm.ReadSerializationFileBinary( filename )
#        Min.Load( eca )
#        del(Min)
#    else:
#        print "ReadData >> ",filename,"is not exist"
#    return eca
#
#############################################################################

def SetResolutionData(ec):
    """
    @param ec      (ElementContainer)
    @retval None
    """
    filenameECbin = "/usr/local/mlf/DNA/tmp/res.dat"
    flug = os.path.exists(filenameECbin)

    if flug:
           os.rename(filenameECbin,filenameECbin+"_bk1" )
    j=0
    f = open(filenameECbin, "w")
    xbin = ec.PutX()
    ybin = ec.PutY()
    ebin = ec.PutE()
    hh= ec.PutHeader()
    flag = ec.PutHeader().CheckKey("CONVOLRANGE")==1

    if flag:
        xr=ec.PutHeader().PutDoubleVector("CONVOLRANGE")
        for i in range(len(ybin)):
            if xr[0]<=(xbin[i]+xbin[i+1])*0.5 and (xbin[i]+xbin[i+1])*0.5<=xr[1]:
                f.write(str(xbin[i])+" "+str(ybin[i])+" "+str(ebin[i])+"\n")
                j=i
        f.write(str(xbin[j+1]))

    else:
        for i in range(len(ybin)):
            f.write(str(xbin[i])+" "+str(ybin[i])+" "+str(ebin[i])+"\n")
        f.write(str(xbin[i+1]))

    f.close()

    return

#############################################################################

def PeakFitBase(ec,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug,useWeight):
    """
    PeakFitting script by using Levmar, AS Fit
    @param ec    (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param link_value      (string)
    @param useDomain       (bool)
    @param NumOfItteration (int)
    @param plotflug        (bool)
    @param useWeight       (bool)
    """
    x_bin = ec.PutX()
    binwidth = (x_bin[1]-x_bin[0])

    param_init_string = param.split(",")
    param_ub_string   = param_ub.split(",")
    param_lb_string   = param_lb.split(",")
    link_list_string=link_value.split(",")
    fit_region_string = fit_region.split(",")
    xbin_start = float(fit_region_string[0])
    xbin_end = float(fit_region_string[1])

    param_init = []
    param_ub = []
    param_lb = []
    link_list =[]

    numOfParam=len(param_init_string)

    for i in range(numOfParam):
         param_init.append(float(param_init_string[i]))
         if i< len(param_ub_string):
             param_ub.append(float(param_ub_string[i]))
         if i< len(param_lb_string):
             param_lb.append(float(param_lb_string[i]))
         if i< len(link_list_string):
            if link_list_string[i].isdigit():
                 link_list.append(int(link_list_string[i]))

    if len(link_list)!=numOfParam:
        link_list=range(1,numOfParam+1)

    op = ml.AdvPeakFit(ec,ml.NEW_LEVMAR)
    op.setDefaultParam()
    if useDomain:
         op.setParam(ml.AdvNewLevmar.CONSTRAIN,          ml.AdvNewLevmar.BOX)
    else:
         op.setParam(ml.AdvNewLevmar.CONSTRAIN,          ml.AdvNewLevmar.NO_CONSTRAIN)

    useDiffMode = useJudgementConvolution(function)
    op.setParam(ml.AdvNewLevmar.USE_NUMERICAL_DIFF, useDiffMode)

    op.setParam(ml.AdvNewLevmar.DIFF_METHOD,        ml.AdvNewLevmar.FORWARD)
    op.setParam(ml.AdvNewLevmar.USE_DATA_WEIGHTS,   useWeight)

    #PeakFitInfoPath=os.path.join("/usr","local","mlf","DNA","ana","xml","PeakfitInfo.xml")
    PeakFitInfoPath = SPD.CheckForInput("xml","PeakfitInfo.xml")
    if PeakFitInfoPath=="":
        raise UserWarning("PeakFitBase >> Not found xml/PeakfitInfo.xml")
    if os.path.exists(PeakFitInfoPath):

        xml = ElementTree(file=PeakFitInfoPath)
        xmlRoot = xml.getroot()

        ResiduErrThresh=xmlRoot.find('RESIDU_ERR_THRESH').text
        op.setParam(ml.AdvNewLevmar.RESIDU_ERR_THRESH, float(ResiduErrThresh))
        GradientThresh=xmlRoot.find('GRADIENT_THRESH').text
        op.setParam(ml.AdvNewLevmar.GRADIENT_THRESH, float(GradientThresh))
        PapramDiffThresh=xmlRoot.find('PARAM_DIFF_THRESH').text
        op.setParam(ml.AdvNewLevmar.PARAM_DIFF_THRESH, float(PapramDiffThresh))
        ScalingFactor=xmlRoot.find('SCALING_FACTOR').text
        op.setParam(ml.AdvNewLevmar.SCALING_FACTOR, float(ScalingFactor))
        NumOfItteration=xmlRoot.find('MAX_ITTERATIONS').text

    op.setParam(ml.AdvNewLevmar.MAX_ITERATIONS,     int(NumOfItteration))
    op.setParam(ml.AdvNewLevmar.OUTPUT_INTERVAL,    int(int(NumOfItteration)*0.1))
    op.setParam(ml.AdvNewLevmar.FUNCTIONS, function)
    op.setParam(ml.AdvNewLevmar.PARAMETER_VALUES, param_init)
    op.setParam(ml.AdvNewLevmar.LOWER_BOUNDS, param_lb)
    op.setParam(ml.AdvNewLevmar.UPPER_BOUNDS, param_ub)
    op.setParam(ml.AdvNewLevmar.LINK_IDS, link_list)

    i1 = getBinNumType2(x_bin,binwidth,xbin_start)
    i2 = getBinNumType2(x_bin,binwidth,xbin_end)
    if i1==i2:
        if i1==0:
            i2=i1+1
        else:
            i1=i2-1  
    op.setDomain(int(i1),int(i2))
    op.fit()
    
    while op.isFitting():
        time.sleep(0.5)

    op.eval()

    result=op.getResult()
    fittedParam=op.getFittedParam()

    LatestConvStat=op.getLatestConvergenceStat()
    chi = LatestConvStat.getDouble(ml.AdvNewLevmar.RESIDUAL_ERR_NORM)
     
    s=fittedParam.getVectorSize(ml.AdvNewLevmar.PARAMETER_VALUES)
    param_result = []
    param_result_error = []
    for i in range(s):
        param_result.append(fittedParam.getDouble(ml.AdvNewLevmar.PARAMETER_VALUES,i))
        param_result_error.append(fittedParam.getDouble(ml.AdvNewLevmar.PARAM_ERRORS,i))

    if plotflug:
        ShowResultPlot(result,ec)
        ShowResultParam(fittedParam,chi)

    hh = ec.PutHeaderPointer()
    for i in range(len(param_result)):
        HeaderOverWrite(hh,"FittingParam"+str(i),param_result[i])
        HeaderOverWrite(hh,"FittingParamErr"+str(i),param_result_error[i])
    HeaderOverWrite(hh,"ChiSq",chi)
    HeaderOverWrite(hh,"FittingFunc",function)
    result.InputHeader(hh)

    return param_result,param_result_error,chi,result

#############################################################################

def PeakFit(ec,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar, AS Fit
    @param ec    (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param link_value      (string)
    @param useDomain       (bool)
    @param NumOfItteration (int)
    @param plotflug        (bool)
    """
    useWeight=True
    return PeakFitBase(ec,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug,useWeight)

#############################################################################

def PeakFitEC(ec,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar, AS Fit
    @param ec    (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param link_value      (string)
    @param useDomain       (bool)
    @param NumOfItteration (int)
    @param plotflug        (bool)
    """
    param_result,param_result_error,chi,result= PeakFit(ec,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug)
    #each_comp_ec = getFittingEachFunction(function,param_result,ec)
    return result

#############################################################################

def PeakConvFitEC(ecDet,ecRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar, AS Fit
    @param ecDet           (ElementContainer)
    @param ecRes           (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param link_value      (string)
    @param useDomain       (bool)
    @param NumOfItteration (int)
    @param plotflug        (bool)
    """
    SetResolutionData(ecRes)
    p1 = listParam2Param(param,0)
    p2 = listParam2Param(param_ub,0)
    p3 = listParam2Param(param_lb,0)
    lk = listParam2Param(link_value,0)

    param_result,param_result_error,chi,result= PeakFit(ecDet,function,p1,p2,p3,lk,fit_region,useDomain,NumOfItteration,plotflug)
    #each_comp_ec = getFittingEachFunction(function,param_result,ec)
    return result

#############################################################################

def PeakFitECABase(ecaDet,ecaRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug,useConv):
    """
    PeakFitting script by using Levmar, AS Fit
    @param ecaDet          (ElementContainerArray)
    @param ecaRes          (ElementContainerArray)
    @param function        (string)
    @param param           (list/string)
    @param param_lb        (list/string)
    @param param_ub        (list/string)
    @param link_value      (string)
    @param useDomain       (bool)
    @param NumOfItteration (int)
    @param plotflug        (bool)
    @param useConv         (bool)
    @retval result_eca (ElementContainerArray)
    """
    eca_size = ecaDet.PutSize()
    if plotflug:
        p=mp.MPlot()
        p.SetTraceNum(2)

    param_result_list = []
    param_result_error_list = []
    chi_list = []
    result_eca = mm.ElementContainerArray()
    #each_comp_eca = mm.ElementContainerArray()
    #PeakFitStatusPath=os.path.join("/usr","local","mlf","DNA","ana","xml","status")
    PeakFitStatusPath=SPD.CheckForInput( "xml","status" )
    if PeakFitStatusPath=="":
        raise UserWarning("PeakFitECABase >> Not found xml/status")
    os.system("echo \""+str(1)+"\">"+PeakFitStatusPath)
    fitStatus=True
    k=0
    for i in range(eca_size):
        #os.system("echo \""+str(i)+"\">log.txt")
        if os.path.exists(PeakFitStatusPath):
            f=open(PeakFitStatusPath, "r")
            for line in f:
                if "0" in line:
                    fitStatus=False
            f.close()

        p1 = listParam2Param(param,i)
        p2 = listParam2Param(param_ub,i)
        p3 = listParam2Param(param_lb,i)
        lk = listParam2Param(link_value,i)
        if useConv:
            if i<ecaRes.PutSize():
                SetResolutionData(ecaRes(i))

        hh0 = ecaDet(i).PutHeaderPointer()
        fitFlug = False
        if (hh0.CheckKey("FIT")==1):
            if (hh0.PutString("FIT") == "t" or hh0.PutString("FIT") == "T") and fitStatus:
                if plotflug:
                    p.AddData(ecaDet(i))
                #print " >> ",i+1," / ",eca_size, "start Fitting process"
                #print ""
                param_result,param_result_error,chi,result = PeakFit(ecaDet(i),function,p1,p2,p3,lk,fit_region,useDomain,NumOfItteration,False)
                #each_comp_ec = getFittingEachFunction(function,param_result,ecaDet(i))
                #print " >> ",i+1," / ",eca_size, "end Fitting process"
                #print ""
                if plotflug:
                    p.AddData(result)
                    p.SetPlotParam(k*2+1,mc="b",lc="b") # data
                    p.SetPlotParam(k*2+2,ls="-",lw=8,mk="",eb=False,lc="r") # result
                    p.ChangePage(i+2)

                param_result_list.append(param_result)
                param_result_error_list.append(param_result_error)
                chi_list.append(chi)
                #hh = mm.HeaderBase()
                hh = ecaDet(i).PutHeaderPointer()
                for i in range(len(param_result)):
                    HeaderOverWrite(hh,"FittingParam"+str(i),param_result[i])
                    HeaderOverWrite(hh,"FittingParamErr"+str(i),param_result_error[i])
                HeaderOverWrite(hh,"ChiSq",0)
                HeaderOverWrite(hh,"FittingFunc",function)
                result.InputHeader(hh)
                result_eca.Add(result)
                #each_comp_eca.Add(each_comp_ec)
                k=k+1

            else:
                paramForNullData=p1.split(",")
                hh = ecaDet(i).PutHeaderPointer()
                for i in range(len(paramForNullData)):
                    HeaderOverWrite(hh,"FittingParam"+str(i),paramForNullData[i])
                    HeaderOverWrite(hh,"FittingParamErr"+str(i),0)
                HeaderOverWrite(hh,"ChiSq",0)
                HeaderOverWrite(hh,"FittingFunc",function)
                HeaderOverWrite(hh,"FIT","F")
                result = nullEC
                result.InputHeader(hh)

                result_eca.Add(result)
                chi_list.append(0)
                vac = []
                param_result_list.append(vac)
                param_result_error_list.append(vac)

        elif (hh0.CheckKey("FIT")==0):
            param_result,param_result_error,chi,result = PeakFit(ecaDet(i),function,p1,p2,p3,lk,fit_region,useDomain,NumOfItteration,False)
            #each_comp_ec = getFittingEachFunction(function,param_result,ecaDet(i))
            param_result_list.append(param_result)
            param_result_error_list.append(param_result_error)
            chi_list.append(chi)
            for i in range(len(param_result)):
                HeaderOverWrite(hh,"FittingParam"+str(i),param_result[i])
                HeaderOverWrite(hh,"FittingParamErr"+str(i),param_result_error[i])
            HeaderOverWrite(hh,"ChiSq",0)
            HeaderOverWrite(hh,"FittingFunc",function)
            hh = ecaDet(i).PutHeaderPointer()

            result.InputHeader(hh)
            result_eca.Add(result)
            #each_comp_eca.Add(each_comp_ec)

    fit_flug_all=False
    for i in range(eca_size):
        fit_flug =False

        hh2 = ecaDet(i).PutHeaderPointer()
        if (hh2.CheckKey("FIT")==1):
            if (hh2.PutString("FIT") == "t" or hh2.PutString("FIT") == "T"):
                fit_flug =True
        elif (hh2.CheckKey("FIT")==0):
                fit_flug =True
        fit_flug_all = fit_flug_all or fit_flug
        if fit_flug:

            print("##############################################################")
            print(" Fit Result ",i+1," / ",eca_size)

            if (hh2.CheckKey("XRANGE")==1):
                XRange=hh2.PutDoubleVector("XRANGE")
                print("    Q region is",XRange[0]," ------ ",XRange[1])
                print("    chiSq = ",chi_list[i])
                for j in range(len(param_result)):
                    print("    param%i\t%.2e\t::: param%i Error\t%.2e"%(j,param_result_list[i][j],j,param_result_error_list[i][j]))
            #else:
            #    print "    fitting is not performed."
    if fit_flug_all:
        print("##############################################################")
 
    return result_eca

#############################################################################

def PeakFitECA(ecaDet,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar, AS Fit
    @param ecaDet          (ElementContainerArray)
    @param function        (string)
    @param param           (list/string)
    @param param_lb        (list/string)
    @param param_ub        (list/string)
    @param link_value      (string)
    @param useDomain       (bool)
    @param NumOfItteration (int)
    @param plotflug        (bool)
    """
    useConv = False
    ecaRes=mm.ElementContainerArray()

    return PeakFitECABase(ecaDet,ecaRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug,useConv)

#############################################################################

def PeakConvFitECA(ecaDet,ecaRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar, AS Fit
    @param ecaDet          (ElementContainerArray)
    @param ecaRes          (ElementContainerArray)
    @param function        (string)
    @param param           (list/string)
    @param param_lb        (list/string)
    @param param_ub        (list/string)
    @param link_value      (string)
    @param useDomain       (bool)
    @param NumOfItteration (int)
    @param plotflug        (bool)
    """
    useConv = True

    return PeakFitECABase(ecaDet,ecaRes,function,param,param_ub,param_lb,link_value,fit_region,useDomain,NumOfItteration,plotflug,useConv)

#############################################################################

def listParam2Param(param,num):
    """
    listPrameter => Parameter
    """
    if(type(param)==list):
        if num<=len(param)-1:
            p1=param[num]
        else:
            p1=param[len(param)-1]
    else:
        p1=param

    return p1

#############################################################################

def PeakFitNotUseWeight(ec,function,param,param_ub,param_lb,fit_region,useDomain,NumOfItteration,plotflug):
    """
    PeakFitting script by using Levmar, AS Fit
    @param ec    (ElementContainer)
    @param function        (string)
    @param param           (string)
    @param param_lb        (string)
    @param param_ub        (string)
    @param useDomain       (bool)
    @param NumOfItteration (int)
    @param plotflug        (bool)
    """
    useWeight=False
    return PeakFitBase(ec,function,param,param_ub,param_lb,fit_region,useDomain,NumOfItteration,plotflug,useWeight)

#############################################################################

def useJudgementConvolution(function):
    """
    @param function        (string)
    @param flug            (bool)
    """
    flugList = [function.find(c) for c in convList]

    flug = False
    for i in flugList:
        if i != -1:
            flug = True
    if flug:
        #print flugList
        #print "use Numerical Diffrential(Convolution)"
        #print ""
        pass

    return flug

#############################################################################

def ShowResultPlot(ec1,ec2):
    """
    @param ec1        (ElementContainer)
    @param ec2        (ElementContainer)
    """
    p = mp.MPlot()
    p.AddData(ec1)
    p.SetPlotParam(1,ls="-",lw=8,mk="",eb=False)
    p.AddData(ec2)

#############################################################################

def getBinNum(x_bin,binwidth,xbin_num):
    """
    """
    i1 = ( xbin_num - x_bin[0] ) / binwidth
    if i1 < 0 :
        i1 = 0
    if i1 >= x_bin.size():
        i1 = x_bin.size() - 2

    return i1

#############################################################################

def getBinNumType2(x_bin,binwidth,xbin_num):
    """
    """
    iout=0
    for i in range(x_bin.size()-1):
        if (x_bin[i]+x_bin[i+1])*0.5 < xbin_num:
            iout = i

    return iout

#############################################################################

def ShowResultParam(fittedParam,chi):
    """
    """
    s=fittedParam.getVectorSize(ml.AdvNewLevmar.PARAMETER_VALUES)
    param_result = []
    param_result_error = []
    print("##############################################################")
    print(" Fit Result ")
    print("    chiSq = ",chi)
    for i in range(s):
        pr = fittedParam.getDouble(ml.AdvNewLevmar.PARAMETER_VALUES,i)
        pre = fittedParam.getDouble(ml.AdvNewLevmar.PARAM_ERRORS,i)
        param_result.append(pr)
        param_result_error.append(pre)
        print("param%i\t%.2e\t::: param%i Error\t%.2e"%(i,pr,i,pre))
    print("##############################################################")

#############################################################################

def SetInitialParam(paramInit):
    """
    @param    paramInit(string)
    @retval   paramInit(string)
    """
    return paramInit

#############################################################################

def SetFunction(function):
    """
    @param    function (string)
    @retval   function (string)
    """
    return function

#############################################################################

def getFunction(function,paramInit,initial_bin,final_bin,delta_bin):
    """
    @param    function (string)
    @param    paramInit(string)
    @retval   dat (ElementContainer/ElementContenerArray)
    """
    ybin=[]
    ebin=[]
    xbin=[]
    ibin=[]
    num_of_bin=int(math.ceil(math.fabs((initial_bin - final_bin)/delta_bin)))

    for i in range(num_of_bin):
        ibin.append(initial_bin+i*delta_bin)

    if type(paramInit)!=list:
        paramStrList = paramInit.split(",")
        param = [float(c) for c in paramStrList]

        func = ml.AdvFuncComb(function,param)
        for i in ibin:
            ybin.append(func.eval(i))
            ebin.append(0)
            xbin.append(i-delta_bin/2)
        xbin.append(final_bin+delta_bin/2)

        dat=mm.ElementContainer()
        dat.Add("x",xbin)
        dat.Add("y",ybin)
        dat.Add("e",ebin)
        dat.SetKeys("x","y","e")
        return dat

    if type(paramInit)==list:
        paramlist_size = len(paramInit)
        eca=mm.ElementContainerArray()
        for j in range(paramlist_size):
            paramStrList = paramInit[j].split(",")
            param = [float(c) for c in paramStrList]

            func = ml.AdvFuncComb(function,param)
            ybin=[]
            ebin=[]
            xbin=[]
            for i in ibin:
                ybin.append(func.eval(i))
                ebin.append(0)
                xbin.append(i-delta_bin/2)
            xbin.append(final_bin+delta_bin/2)

            ec=mm.ElementContainer()
            ec.Add("x",xbin)
            ec.Add("y",ybin)
            ec.Add("e",ebin)
            ec.SetKeys("x","y","e")
            eca.Add(ec)

        return eca

#############################################################################

def getFittingFunction(function,paramInit,ec):
    """
    @param    function (string)
    @param    paramInit(string)
    @param    ec       (ElementCOntainer)
    @retval   dat (ElementContainer/ElementContenerArray)
    """
    ybin=[]
    ebin=[]
    ecX=ec.PutX()
    xbin = [ecX[i] for i in range(ecX.size())]
    if type(paramInit)!=list:
        paramStrList = paramInit.split(",")
        param = [float(c) for c in paramStrList]

        func = ml.AdvFuncComb(function,param)
        for i in range(len(xbin)-1):
            ybin.append(func.eval( (ecX[i]+ecX[i+1])*0.5 ))
            ebin.append(0)

        dat=mm.ElementContainer()
        dat.Add("x",xbin)
        dat.Add("y",ybin)
        dat.Add("e",ebin)
        dat.SetKeys("x","y","e")

        hh = ec.PutHeaderPointer()
        HeaderOverWrite(hh,"FittingFunc",function)
        dat.InputHeader(hh)

        return dat

    if type(paramInit)==list:
        paramlist_size = len(paramInit)
        eca=mm.ElementContainerArray()
        for j in range(paramlist_size):
            paramStrList = paramInit[j].split(",")
            param = [float(c) for c in paramStrList]

            func = ml.AdvFuncComb(function,param)
            ybin=[]
            ebin=[]
            for i in range(len(xbin)-1):
                ybin.append(func.eval((ecX[i]+ecX[i+1])*0.5))
                ebin.append(0)

            e0c=mm.ElementContainer()
            ec0.Add("x",xbin)
            ec0.Add("y",ybin)
            ec0.Add("e",ebin)
            ec0.SetKeys("x","y","e")

            hh = ec.PutHeaderPointer()
            HeaderOverWrite(hh,"FittingFunc",function)
            ec0.InputHeader(hh)
            eca.Add(ec0)

        return eca

#############################################################################
#
#def getFittingFunction(function,paramInit,ec):
#    """
#    @param    function (string)
#    @param    paramInit(string)
#    @retval   dat (ElementContainer/ElementContenerArray)
#    """
#    DetX = ec.PutX()
#    initial_bin =  DetX[0]
#    final_bin   =  DetX[DetX.size()-1]
#    delta_bin   =  DetX[1]-DetX[0]
#
#    return getFunction(function,paramInit,initial_bin,final_bin,delta_bin)
#
#############################################################################
def getEachFunction(func,param,initial_bin,final_bin,delta_bin):
    """
    """
    ybin=[]
    ebin=[]
    xbin=[]
    ibin=[]
    # calc ibin value
    num_of_bin=int(math.ceil(math.fabs((initial_bin - final_bin)/delta_bin)))
    for i in range(num_of_bin):
        ibin.append(initial_bin+i*delta_bin)

    FuncComp = mm.ElementContainerArray()
    func_list=func.split()

    # calc number of required parameter of each functions
    NumOfParamList = []
    for i in range(len(func_list)):
        func = ml.AdvFuncComb(func_list[i])
        NumOfParam = func.getNumberOfRequiredParam()
        NumOfParamList.append(NumOfParam)

    # calc parameter of each functions
    paramList = []
    l = 0
    for j in range(len(func_list)):
        paramList0 = []
        for k in range(NumOfParamList[j]):
            paramList0.append(float(param[l]))
            l += 1
        paramList.append(paramList0)

    # eval each function and fill ElementContainerArray
    for j in range(len(func_list)):
        func = ml.AdvFuncComb(func_list[j],paramList[j])
        ybin=[]
        ebin=[]
        xbin=[]
        for i in ibin:
            ybin.append(func.eval(i))
            ebin.append(0)
            xbin.append(i-delta_bin/2)
        xbin.append(final_bin+delta_bin/2)
        ec=mm.ElementContainer()
        ec.Add("x",xbin)

        if(ybin[0]!=ybin[0]):
            ec.Add("y",[0]*len(ibin))
            ec.Add("e",[0]*len(ibin))
        if(ybin[0]==ybin[0]):
            ec.Add("y",ybin)
            ec.Add("e",ebin)
        ec.SetKeys("x","y","e")
        FuncComp.Add(ec)

    return FuncComp

#############################################################################
def getFittingEachFunction(func,param,ec):
    """
    """
    ybin=[]
    ebin=[]
    ecX=ec.PutX()
    xbin = [ecX[i] for i in range(ecX.size())]

    FuncComp = mm.ElementContainerArray()
    func_list=func.split()

    # calc number of required parameter of each functions
    NumOfParamList = []
    for i in range(len(func_list)):
        func = ml.AdvFuncComb(func_list[i])
        NumOfParam = func.getNumberOfRequiredParam()
        NumOfParamList.append(NumOfParam)

    # calc parameter of each functions
    paramList = []
    l = 0
    for j in range(len(func_list)):
        paramList0 = []
        for k in range(NumOfParamList[j]):
            paramList0.append(float(param[l]))
            l += 1
        paramList.append(paramList0)

    # eval each function and fill ElementContainerArray
    for j in range(len(func_list)):
        func = ml.AdvFuncComb(func_list[j],paramList[j])

        ybin=[]
        ebin=[]
        for i in range(len(xbin)-1):
            ybin.append(func.eval((xbin[i]+xbin[i+1])*0.5))
            ebin.append(0)
        ec=mm.ElementContainer()
        ec.Add("x",xbin)

        if(ybin[0]!=ybin[0]):
            ec.Add("y",[0]*(len(xbin)-1))
            ec.Add("e",[0]*(len(xbin)-1))
        if(ybin[0]==ybin[0]):
            ec.Add("y",ybin)
            ec.Add("e",ebin)
        ec.SetKeys("x","y","e")

        FuncComp.Add(ec)

    return FuncComp

#############################################################################
#
#def getFittingEachFunction(function,paramInit,ec):
#    """
#    @param    function (string)
#    @param    paramInit(string)
#    @retval   dat (ElementContainer/ElementContenerArray)
#    """
#    DetX = ec.PutX()
#    initial_bin =  DetX[0]
#    final_bin   =  DetX[DetX.size()-1]
#    delta_bin   =  DetX[1]-DetX[0]
#
#    return getEachFunction(function,paramInit,initial_bin,final_bin,delta_bin)
#
#############################################################################

def getResidualDistribution(ec,result):
    """
    @param    ec     (ElementContainer/ElementContainerArray)
    @param    result (ElementContainer/ElementContainerArray)
    @retval   ecOut  (ElementContainer/ElementContainerArray)
    """
    if type(result)==mm.ElementContainer:
        if type(ec)==mm.ElementContainer:
            ecIn = ec
        if type(ec)==mm.ElementContainerArray:
            ecIn = ec(0)
        ecOut = getResidualDistributionBase(ecIn,result)

        return ecOut

    ecOut = mm.ElementContainerArray()
    if type(result)==mm.ElementContainerArray:
        ecaOut = mm.ElementContainerArray()
        for i in range(result.PutSize()):
            if type(ec)==mm.ElementContainer:
                ecIn = ec
            if type(ec)==mm.ElementContainerArray:
                ecIn = ec(i)
            ecOut_buf = getResidualDistributionBase(ecIn,result(i))
            ecOut.Add(ecOut_buf)

        return ecOut

#############################################################################

def getResidualDistributionBase(ecIn,result):
    """
    @param    ecIn     (ElementContainer)
    @param    result (ElementContainer)
    @retval   ecOut  (ElementContainer)
    """
    ecOut = mm.ElementContainer()
    ecIn_xbin_buf = ecIn.PutX()
    ecIn_ybin_buf = ecIn.PutY()
    ecIn_ebin_buf = ecIn.PutE()

    result_xbin = result.PutX()
    result_ybin = result.PutY()
    result_ebin = result.PutE()
    strt_bin = 0
    end_bin  = 0
    if (ecIn_xbin_buf.size() == result_xbin.size()):
        ecIn_xbin = ecIn_xbin_buf
        ecIn_ybin = ecIn_ybin_buf
        ecIn_ebin = ecIn_ebin_buf
    if (ecIn_xbin_buf.size() != result_xbin.size()):
        for i in range(ecIn_xbin_buf.size()):
            #print "#",result_xbin[0],ecIn_xbin_buf[i],result_xbin[result_xbin.size()-1]
            if (abs(result_xbin[0] - ecIn_xbin_buf[i]) < 1e-10):
                start_bin = i
            if (abs(result_xbin[result_xbin.size()-1] - ecIn_xbin_buf[i]) < 1e-10):
                end_bin = i
        #if start_bin == 0 or end_bin ==0: #20120921 "start_bin ==0"condition is removed
        if end_bin == 0:
            print("end_bin == 0")
            return ecOut

        ecIn_xbin = []
        ecIn_ybin = []
        ecIn_ebin = []
        for i in range(start_bin,end_bin):
            ecIn_xbin.append(ecIn_xbin_buf[i])
            ecIn_ybin.append(ecIn_ybin_buf[i])
            ecIn_ebin.append(ecIn_ebin_buf[i])
        ecIn_xbin.append(ecIn_xbin_buf[end_bin])

    ecOut_xbin=result_xbin
    ecOut_ybin=[]
    ecOut_ebin=[]
    for i in range(result_xbin.size()-1):
        if i<len(ecIn_ebin):
            if ecIn_ebin[i] !=0:
                ecOut_ybin.append((ecIn_ybin[i]-result_ybin[i])/ecIn_ebin[i])
            if ecIn_ebin[i] ==0:
                ecOut_ybin.append(0)
            ecOut_ebin.append(0)
        else:
            ecOut_ybin.append(0)
            ecOut_ebin.append(0)

    ecOut.Add("x",ecOut_xbin)
    ecOut.Add("y",ecOut_ybin)
    ecOut.Add("e",ecOut_ebin)
    ecOut.SetKeys("x","y","e")

    return ecOut

#############################################################################

def PeakSearchSimple(ec,window_width,num_of_peak):
    """
    PeakSearch script , AS PeakSearch
    @param ec (ElementContainer)
    @param window_width (int)
    @param num_of_peak  (int)
    @retval outputPeakDataText (string)
    """
    pk = ml.AdvPeakSearch(ec,ml.MOVING_AVERAGE)
    pk.setDefaultParam()
    pk.setParam(ml.AdvMovingAverage.WINDOW_WIDTH,window_width)
    window_upper = int(math.ceil(window_width/2))
    pk.setParam(ml.AdvMovingAverage.WINDOW_UPPER,window_upper)
    #pk.setDomain(lower_bin,upper_bin)

    pk.execute()
    pl=pk.getPeaks()
    v = pl.toVector()
    #pl.Dump()

    peakData=[]
    for i in range(v.size()):
        if i%3==0:
            eachPeakData=[]
            eachPeakData.append(float(v[i]))
        if i%3==1:
            eachPeakData.append(float(v[i]))
        if i%3==2:
            eachPeakData.append(float(v[i]))
            peakData.append(eachPeakData)
    peakData.sort(cmp = lambda x , y: cmp(-x[0],-y[0]) )

    outputPeakData=[]
    outputPeakDataText=""
    #print"[height,position,width]"
    for i in range(int(num_of_peak)):
        #outputPeakData.append(peakData[i])
        outputPeakData.append([ peakData[i][0]*peakData[i][2]*math.pi,peakData[i][1],peakData[i][2] ])
        #print outputPeakData[i]
        if i != int(num_of_peak)-1:
            outputPeakDataText+="%.2f,%.2f,%.2f,"%(outputPeakData[i][0],outputPeakData[i][1],outputPeakData[i][2])
            #print "%.2f,%.2f,%.2f,"%(outputPeakData[i][0],outputPeakData[i][1],outputPeakData[i][2]),
        if i == int(num_of_peak)-1:
            outputPeakDataText+="%.2f,%.2f,%.2f"%(outputPeakData[i][0],outputPeakData[i][1],outputPeakData[i][2])
            #print "%.2f,%.2f,%.2f"%(outputPeakData[i][0],outputPeakData[i][1],outputPeakData[i][2])
    return outputPeakDataText

#############################################################################

def HeaderOverWrite(hh,name,val):
        if (hh.CheckKey(name)==0):
            hh.Add(name,val)
        elif (hh.CheckKey(name)==1):
            hh.OverWrite(name,val)

#############################################################################
#ReadData
#ecaDet = ReadDataECA("ECA_637miya.bst")
#ecaRes = ReadDataECA("ECA_647miya.bst")
#p = mp2.M2Plot(ecaDet)

##QEMask
#QEfilename = "QEMask.txt"
#DoQEMask(ecaDet,QEfilename)

# Q-Slice test-1
#lowerQ = 0.0
#upperQ = 2.0
#ecRes = SliceQE(ecaRes,lowerQ,upperQ)

# Q-Slice test-2
#initialQ =  0.0
#finalQ =    1.0
#deltaQ =    0.1
#eca = SliceQEall(ecaDet,initialQ,finalQ,deltaQ)

# Q-Slice test-3
#Q_list_string = "0.0,0.1,0.3,0.5,0.7,0.9,1.3,1.6,2.0"
#ecaDetS = SliceQEbyText(ecaDet,Q_list_string)
#ecaResS = SliceQEbyText(ecaRes,Q_list_string)

#SelectFittingQbin(eca,"t")
#SelectFittingQbin(ecaDetS,"t,t,t,t,t,t,t,f")
#SelectFittingQbin(ecaDetS,"f,t,t,t,t,f,f,f")

#CheckQbin(ecaDetS)

## set Fitting Parameter
#fit_region = "-0.1,0.1"
#useDomain = False
#useDomain = True
#NumOfItteration = 1000
#plotflug = True

## peakSearch
#peakData = PeakSearchSimple(ecaDetS(3),1,1)

#function = SetFunction("lc dc")

#param set common
#param       = SetInitialParam("40,0.0,2e-2,0.0,8000") #0 lc dc

#lc dc
#param       = SetInitialParam("10 ,   0.0, 2e-2,   0.0,8000") #1,2,3,4,--5,6,7 lc dc
#param_ub    = SetInitialParam("100, 0.001,  1,   0.001,80000")
#param_lb    = SetInitialParam("1  ,-0.001, 1e-4,-0.001,  100")

#param set each ec
#param = []
#param.append(SetInitialParam("10 ,0,5e-2"))
#param.append(SetInitialParam("10 ,0,5e-2"))
#param.append(SetInitialParam("200,0,5e-2"))
#param.append(SetInitialParam("80,0,5e-2"))
#param.append(SetInitialParam("30 ,0,5e-2"))
#param.append(SetInitialParam("20 ,0,5e-2"))

## fitting single
#result   = PeakConvFitEC(ecaDetS(2),ecaResS(2),function,param,param_ub,param_lb,fit_region,useDomain,NumOfItteration,plotflug)

## fitting multi
# QENS_Base de NumOfItteration,plotflug ha settei
#result = PeakConvFitECA(ecaDetS,ecaResS,function,param,param_ub,param_lb,fit_region,useDomain,NumOfItteration,plotflug)

## get ElementContainer of fitted function
#ecInit   = getFittingFunction(function,param,ecaDetS(0))

## get residual data(EC) between original-data and fitting-result-function 
#residual = getResidualDistribution(ecInit,result)

## plot fitting parameter 
#ecP = getFittingParameterEC(result,0,"Q")
#ecP = getFittingParameterEC(result,"x","Q")

## plot EISF 
#ecP=getEisfEC(result,"0,2","4","Q2")
#ecP=getEisfEC(result,"4,1,2","0,4","Q2")
#p=mp.MPlot()
#p.AddData(ecP)



#!/usr/bin/python

import Manyo
import ParamSet

v = Manyo.MakeDoubleVector()
v.push_back(1.0)
v.push_back(2.0)
v.push_back(3.0)

v1 = Manyo.MakeDoubleVector()
v1.push_back(1.0)
v1.push_back(2.0)
v1.push_back(3.0)
v1.push_back(4.0)
v1.push_back(5.0)
v1.push_back(6.0)

pc = ParamSet.PeakData()
pc.add(ParamSet.PEAK, -3.5, 100000.0, 3.5, -7.0, 0.0)
pc.add(ParamSet.PEAK,  3.5, 100000.0, 3.5,  0.0, 7.0)
print "number of peaks", pc.size()
pc.Dump()

param = ParamSet.ParamSet()

#    boolean value
param.add("uniform knots", 1)
#param.add("uniform knots", 1)  # error will occurs
param.add("unformatted",   0)

#    Int4 type value
param.add("window width", 1)
param.add("upper bound",  1)
param.add("order",        4)
#param.add("order",        4)   # error will occurs

#    UInt4 type value
#param.add("order", 4)
#param.add("order", 4)    # error will occuers

#    Double type value
param.add("pi", 3.1415926)
#param.add("pi", 3.1415926)    # error will occurs
param.add("base of natural logalithm", 2.7189)

#    vector
param.add("v0", v)
#param.add("v0", v)    # error will occurs

#    PeakData
param.add("peak data", pc)

param.dump()


print "uniforn knots:", param.getUInt4("uniform knots")
print "unformatted  :", param.getUInt4("unformatted")
print "window width :", param.getUInt4("window width")
#print "window upper :", param.getUInt4("window upper")    # error will occurs
print "upper bound  :", param.getUInt4("upper bound")
print "order        :", param.getUInt4("order")

print "pi                       :", param.getDouble("pi")
print "base of natural logalithm:", param.getDouble("base of natural logalithm")
print "gamma                    :", param.getDouble("gamma")

v0 = param.getVector("v0")
for i in range(v0.size()):
    print "v0[", i, "]=", v0[i]
print ""

####    replace    ####
param.replace("uniform knots",  0)
#param.replace("formatted",      1)
param.replace("unformatted",    1)
param.replace("window width",  11)
param.replace("order",         10)

param.replace("pi",           3.0)

param.replace("v0", 2, 4.0)
param.dump()

param.replace("v0", v1)
param.dump()

keyList=param.getKeyList()
print keyList
for i in range(keyList.size()):
    print "key=", keyList[i]



#!/usr/bin/python
# -*- coding: utf-8 -*-

import os
import Manyo as mm
import Manyo.Utsusemi as mu
import ana.Reduction.AnaEnviron as AE
import ana.Reduction.EditWiringFile as ED

import ana.Reduction.EventDataToEcm as EtoE

try:
    ver_ED = float( ED.__version__ )
    if (ver_ED<0.6):
        raise "VersionError","EditDetectorInfo or EventDataToHist is not valid."
except:
    raise "VersionError","EditDetectorInfo or EventDataToHist is not valid."

__version__ = '0.61'

class Histogram(object):
    """
    Load EventData and convert to histogram.
    ver 0.62[141209] add TofMask
    ver 0.61[140630] add LLD
    ver 0.6 [130304] update for Monitor
    ver 0.5 [130213] upgrade
    ver 0.4 [110717] replace EventDataToHist to EventDataToEcm
    ver 0.3 [110104] add control tofDoubleFrameBoundary
    ver 0.2 [101108] can use direct conversion from event data to energy transfer
    ver 0.1 before 101108 original version
    """
    __version__ = '0.5'
    def __init__(self,runNo=[],
                 ConvType=-1,
                 Params=[],
                 DetRange=[],
                 pixelNo=-1,
                 TimeRange=[],
                 envFile="environ_ana.xml",
                 frameInfo=[0,0.0],
                 DetType="PSD",
                 LLD=[],
                 TofMask="",
                 withPulseHeight=False,
                 isDebug=False):
        """
        Constructor and initialization
        
        @param runNo    ([]) Run Number 0:Dummy data
        @param ConvType (int) TOF Conversion Type
        @param Params ([])
        @param DetRange ([])  [first PSD ID, final PSD ID]
        @param pixelNo  (int) the number of pixels on one PSD
        @param TimeRange ([]) [startTime, endTime] [seconds]
        @param envFile  (string) Environ file for event data to histogram
        @param frameInfo ([int,double]) FrameNo and Boundary TOF
        @param DetType  (string) PSD or N2MON
        @param LLD ([]) LLD and HLD for DetRange
        @param TofMask (string) 
        @param withPulseHeight (bool)
        @param isDebug  (bool)
        """
        self.isParallel = False
        
        self.runNo = None
        if type(runNo)==type([]):
            self.runNo = runNo
        elif type(runNo)==type(0):
            self.runNo = [ runNo ]
        else:
            msg = "Histogram >> runNo is invalid."
            raise UserWarning, msg
        
        self.ConvType = ConvType
        #self.isPulseHeight = False
        
        isBadParams = False
        if ((self.ConvType==2) or (self.ConvType==3) or (self.ConvType==12) or (self.ConvType==13) or (self.ConvType==21) or (self.ConvType==25)):
            if len(Params)<3:
                isBadParams = True
        elif (self.ConvType==20):
            if len(Params)<5:
                isBadParams = True
        elif (self.ConvType==22):
            if len(Params)<6:
                isBadParams = True
        else:
            msg = "Invalid ConvType (%d)"%(self.ConvType)
            raise UserWarning, msg
        """
        if (self.ConvType==50):  ## convert to pulse height
            if len(Params)<3:
                isBadParams = True
            else:
                self.isPulseHeight = True
                self.ConvType=2
        """
        if isBadParams:
            msg = "Num of Parameters is invalid for type=%d ("
            for p in Params:
                msg += " %f" % (p)
            msg +=")"
            raise UserWarning, msg

        self.Params = Params
        self.startDet = DetRange[0]
        self.endDet = DetRange[1]
        self.numOfPixelOnDet = pixelNo
        if len(TimeRange)>=2:
            self.SliceEventStart = TimeRange[0]
            self.SliceEventEnd = TimeRange[1]
        else:
            self.SliceEventStart = -1.0
            self.SliceEventEnd = -1.0
        self.envFile = ""
        if envFile.find(",")==-1:
            self.envFile = envFile
        else:
            self.envFile = "environ_ana.xml"
        self.frameInfo = frameInfo
        self.DetType = DetType
        self.LLD = LLD
        self.TofMask = TofMask
        #self.isPulseHeight = isPulseHeight
        self.withPulseHeight = withPulseHeight
        self.isDebug = isDebug
        #self.isDebug =True 
        
        ##self.L1 = EtoE.Def_L1*1000.0 #[mm]
        self.L1 = -1.0
        self.T0shift = 0.0
        self.instCode = os.environ["UTSUSEMI_SYS_NAME"]
        self.CommentHead = "Histogram >>>"
        
        self.DetectorInfo = None
        ana_env = AE.AnaEnviron(self.envFile)
        if ana_env.Status==-1:
            print self.CommentHead+"environ file ("+self.envFile+") is not existed."
            self.envFile = ""
            self.anaEnv = None
            self.Status = -1
            return
        else:
            self.anaEnv = ana_env
            ind = envFile.find(",")
            if self.isDebug:
                print "---================ Check envFile=",envFile,"ind=",ind
            if ind==-1:
                if self.anaEnv.Info["OrgWiringFile"]==None:
                    lastPeriod = 0
                    lastPeriodFile=[]
                    for period in self.anaEnv.Info["FileHistory"]:
                        if (self.runNo[0] >= period[0]) and (self.runNo[0] <= period[1]):
                            self.anaEnv.Info["OrgWiringFile"] = period[2]
                            self.anaEnv.Info["PositioningFile"] = period[3]
                            break
                        if period[1]>=lastPeriod:
                            lastPeriod = period[1]
                            lastPeriodFile=[ period[2], period[3] ]
                    if self.anaEnv.Info["OrgWiringFile"]==None:
                        if len(lastPeriodFile)==0:
                            msg = "Run No is not suitable to choose WiringInfo from environ_ana.xml"
                            raise UserWarning,msg
                        else:
                            self.anaEnv.Info["OrgWiringFile"] = lastPeriodFile[0]
                            self.anaEnv.Info["PositioningFile"] = lastPeriodFile[1]
            else:
                if self.isDebug:
                    print "=================== Changed by envFile"
                self.anaEnv.Info["OrgWiringFile"] = envFile[:ind]
                if envFile[(ind+1):]!="":
                    self.anaEnv.Info["PositioningFile"] = envFile[(ind+1):]
            self.Status = 0
            #self.DetectorInfo = mu.UtsusemiDetectorInfoReader3HePsd( self.anaEnv.Info["PositioningFile"] )
            path_dfile = ""
            if os.path.exists( self.anaEnv.Info["PositioningFile"] ):
                path_dfile = self.anaEnv.Info["PositioningFile"]
            else:
                path_dfile = os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml",self.anaEnv.Info["PositioningFile"])
                if not os.path.exists( path_dfile ):
                    self.Status = -1
                    print "Cannot find DetectorInfo file "+self.anaEnv.Info["PositioningFile"]
                    return
                
            self.DetectorInfo = mu.UtsusemiDetectorInfoReader3HePsd( path_dfile )
            self.L1 = self.DetectorInfo.PutInstL1()
            

    def MakeWiringFile(self):
        if self.anaEnv== None:
            print self.CommentHead+"Set file name of environment file (loadEnvFile)."
            return
        
        path_org_wf = os.path.join(os.environ["UTSUSEMI_USR_DIR"],"ana","xml",self.anaEnv.Info["OrgWiringFile"])
        if not os.path.exists(path_org_wf):
            print self.CommentHead+"ERROR: There is no original wiring file,",path_org_wf
            return -1
        
        return self._MakeWiringFile(path_org_wf,self.anaEnv.Info["SingleMode"])
            

    def _MakeWiringFile(self,path_org_wf,p=None):
        print self.CommentHead+"Read Original WiringInfo ("+path_org_wf+")"
        ed = ED.EditWiringInfo(path_org_wf)
        daq_list=ed.InstInfo.PutContainerIdList()

        for daq_r in daq_list:
            ed.SuspendDaq(daq_r)
        
        new_module_list =[]
        if (self.startDet>=0) and (self.endDet<=ed.MaxDetId):
            for i in range(int(self.startDet),int(self.endDet+1)):
                (flag,daq_i,mod_i) = ed.DetInfoList[i]
                if self.isDebug:
                    print "flag,daq_i,mod_i",flag,daq_i,mod_i
                if (flag!=0):
                    for j in ed.InstInfo.PutContainer(daq_i).PutContainer(mod_i).PutContainerIdList():
                        info = ed.InstInfo.PutContainer(daq_i).PutContainer(mod_i).PutContainer(j) 
                        info.SetStatus(1)
                    if not (daq_i,mod_i) in new_module_list:
                        new_module_list.append( (daq_i,mod_i) )
        else:
            for (daq,mod) in p.Info["modulelist"]:
                if daq==-1:
                    for daq_r in daq_list:
                        ed.ResumeDaq(daq_r)
                    break
                else:
                    ed.ResumeModules(daq,mod)

        ed.SelectByDetType( self.DetType )
        
        if new_module_list !=[]:
            p.Info["modulelist"]=new_module_list
            
        if self.frameInfo[0]!=0:
            ed.set_FrameInfo( self.frameInfo[0],self.frameInfo[1] )
            if self.isDebug:
                print self.CommentHead+("Set FrameNo %d Boundary TOF %f" % (self.frameInfo[0],self.frameInfo[1] ))
        
        if len(self.LLD)==2:
            ed.set_LLD( self.LLD[0], self.LLD[1] )
            
        if (self.TofMask!=""):
            ed.set_TofMask( 0, self.TofMask )
            
        if self.numOfPixelOnDet!=-1:
            ed.ReSetNumOfPixel( 0,ed.MaxDetId, -1, self.numOfPixelOnDet )
            if self.isDebug:
                print self.CommentHead+"Change the number of pixels on one PSD to ",self.numOfPixelOnDet

        if self.ConvType!=-1:
            ptnId = ed.change_TofBinPattern(type=self.ConvType,cont_in=self.Params)
            maskId = -1
            if (self.TofMask!=""):
                maskId = 0
            ed.setall_TofBinInfo(ptnId,0,maskId)
            print self.CommentHead+"ConvType=",self.ConvType,", Params = ",self.Params
        
        p.Info["wiringfile"]="%s_%3s%06d_%06d_%s" % (os.environ["USER"],self.instCode,self.runNo[0],os.getpid(),p.Info["wiringfile"])
        
        if os.environ.has_key("UTSUSEMI_USR_PRIV_HOME"):
            ss = os.path.join(os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","xml",p.Info["wiringfile"])
        elif os.environ.has_key("HOME"):
            ss = os.path.join(os.environ["HOME"],"ana","xml",p.Info["wiringfile"])
        else:
            ss = p.Info["wiringfile"]
        ed.SaveToXml( ss )
        comm = "chmod go+w "+ss
        print self.CommentHead+"Do command = ",comm
        try:
            os.system(comm)
        except:
            print self.CommentHead+" Command missed."

        return 0


    def GetHistogram(self,daq=-1):
        if self.anaEnv==None:
            print self.CommentHead+" There is no information."
            return -1

        if self.instCode != self.anaEnv.Info["InstCode"]:
            print self.CommentHead+" Instrument code is not correct."
            return -1
        
        p=self.anaEnv.Info["SingleMode"]
        
        DAT = EtoE.GetHistogram(eventtype=self.DetType,
                                runnumber=self.runNo,
                                WiringInfo_xml=p.Info["wiringfile"],
                                DetectorInfo_xml=self.anaEnv.Info["PositioningFile"],
                                starttime=self.SliceEventStart,
                                endtime=self.SliceEventEnd,
                                withPH=self.withPulseHeight
                                )
        """
        DAT = EtoE.GetHistogram( self.DetType, self.runNo,
                                 p.Info["wiringfile"],
                                 self.anaEnv.Info["PositioningFile"],
                                 self.SliceEventStart,self.SliceEventEnd )
        """
        if not self.isDebug:
            comm = "rm -rf %s" % (os.path.join(os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","xml",p.Info["wiringfile"]))
            try:
                os.system(comm)
            except:
                print self.CommentHead+" False to delete %s."%(os.path.join(os.environ["UTSUSEMI_USR_PRIV_HOME"],"ana","xml",p.Info["wiringfile"]))
                
        return DAT


###############################################
### Sample for utilization of Histogram class
###############################################
def GetHistDTconst(runNo=-1,startTOF=-1,endTOF=-1,binTOF=-1,startPSD=-1,endPSD=-1,pixelNo=-1,bTOF=0.0,detType="PSD"):
    """
    Load EventData and convert to histogram.
    GetHistogram(runNo=-1,paraFlag=0,startTOF=-1,endTOF=-1,binTOF=-1,startPSD=-1,endPSD=-1,pixelNo=-1,bTOF=0.0)
    @param runNo    (int) Run Number
    @param startTOF (double) top TOF value to be selected
    @param endTOF   (double) end TOF value to be selected
    @param binTOF   (double) bin width of TOF
    @param startPSD (int) first PSD ID
    @param endPSD   (int) final PSD ID
    @param pixelNo  (int) the number of pixels on one PSD
    @param bTOF     (double) boundary tof for double frame mode
    @param detType  (string) PSD or N2MON
    @retval ElementContainerMatrix
    """
    fInfo = [0,0.0]
    if bTOF!=0.0:
        fInfo[0] = int( bTOF/40000.0 )
        fInfo[1] = bTOF - float(fInfo[0])*40000.0

    params = [startTOF, endTOF, binTOF ]
    drange = [startPSD, endPSD]
    trange = [-1.0, -1.0]
    h=Historam(runNo=runNo,
               ConvType=2,
               Params=params,
               DetRange=drange,
               pixelNo=pixelNo,
               TimeRange=trange,
               frameInfo=fInfo,
               DetType=detType,
               )
    
    if h.Status==0:
        h.MakeWiringFile()
        return h.GetHistogram()

def GetHistogramSimple( runNo=-1,wiringFile="WiringFile_org.xml",detectCompFile="DetectorPosi2.xml", startTOF=-1,endTOF=-1,binTOF=-1,startPSD=-1,endPSD=-1,pixelNo=-1,bTOF=0.0 ):
    """
    Load EventData and confert to histogram using given WiringFile and DetectorCompFile.
    GetHistogramSimple( runNo=-1,wiringFile="WiringFile_org.xml",detectCompFile="DetectorPosi2.xml", 
                        startTOF=-1,endTOF=-1,binTOF=-1,startPSD=-1,endPSD=-1,pixelNo=-1,bTOF=0.0
                      )
    @param runNo    (int) Run Number
    @param wiringFile (string) Wiring File name
    @param detectCompFile (string) Detector Components file name
    @param startTOF (double) top TOF value to be selected
    @param endTOF   (double) end TOF value to be selected
    @param binTOF   (double) bin width of TOF
    @param startPSD (int) first PSD ID
    @param endPSD   (int) final PSD ID
    @param pixelNo  (int) the number of pixels on one PSD
    @param bTOF (double) boundary tof for double frame mode
    @retval ElementContainerMatrix
    """
    in_envFile = wiringFile+","+detectCompFile
    in_envFile = in_envFile.replace(" ","")

    fInfo = [0,0.0]

    if bTOF!=0.0:
        fInfo[0] = int( bTOF/40000.0 )
        fInfo[1] = bTOF - float(fInfo[0])*40000.0

    params = [startTOF, endTOF, binTOF ]
    drange = [startPSD, endPSD]
    trange = [-1.0, -1.0]
    h=Histogram(runNo=runNo,
                ConvType=-1,
                Params=params,
                DetRange=drange,
                pixelNo=pixelNo,
                TimeRange=trange,
                envFile=in_envFile,
                frameInfo=fInfo,
                DetType="PSD",
                withPulseHeight=False,
                isDebug=False
                )
    
    if h.Status==0:
        h.MakeWiringFile()
        return h.GetHistogram()

def GetHistogramByTime(runNo=-1,startTOF=-1,endTOF=-1,binTOF=-1,startPSD=-1,endPSD=-1,pixelNo=-1,startTime=-1.0,endTime=-1.0,bTOF=0.0):
    """
    Load EventData and convert to histogram.
    Histogram(runNo,paraFlag,startTOF=-1,endTOF=-1,binTOF=-1,pixelNo=-1,startPSD=-1,endPSD=-1,envFile)
    @param runNo    (int) Run Number
    @param startTOF (double) top TOF value to be selected
    @param endTOF   (double) end TOF value to be selected
    @param binTOF   (double) bin width of TOF
    @param startPSD (int) first PSD ID
    @param endPSD   (int) final PSD ID
    @param pixelNo  (int) the number of pixels on one PSD
    @param startTime (float)
    @param endTime   (float)
    @param bTOF (double) boundary tof for double frame mode
    @retval ElementContainerMatrix
    """
    fInfo = [0,0.0]
    if bTOF!=0.0:
        fInfo[0] = int( bTOF/40000.0 )
        fInfo[1] = bTOF - float(fInfo[0])*40000.0

    params = [startTOF, endTOF, binTOF ]
    drange = [startPSD, endPSD]
    trange = [startTime, endTime]
    h=Histogram(runNo=runNo,
                ConvType=2,
                Params=params,
                DetRange=drange,
                pixelNo=pixelNo,
                TimeRange=trange,
                frameInfo=fInfo,
                DetType="PSD",
                withPulseHeight=False,
                isDebug=False
                )
    
    if h.Status==0:
        h.MakeWiringFile()
        return h.GetHistogram()
    



def GetHistogramOfConv( runNo=-1,
                        ConvType=-1,
                        Params=[],
                        detectorRange=[-1,-1],
                        numOfPixel=-1,
                        timeRange=[-1.0,-1.0],
                        frameInfo=[0,0.0],
                        DetType="PSD"
                        ):
                        
    """
    Load EventData and convert to histogram
    GetHistogramOfConv( runNo=-1,
                        ConvType=-1,
                        Params=[],
                        detectorRange=[-1,-1],
                        numOfPixel=-1,
                        timeRange=[-1.0,-1.0],
                        frameInfo=[0,0.0],
                        DetType="PSD"
                        )
    @param runNo         (int)   run number
    @param ConvType      (int)   Conversion Type 
    @param Params        (list)  parameters required by ConvType
    @param detectorRange (list)  range of detectors [startId, endId]
    @param numOfPixel    (int)   the number of pixels of a PSD
    @param timeRange     (list)  time range [start Time[sec], end Time[sec] ]
    @param frameInfo     (list)  frame info
    @param DetType       (string) "PSD" or "N2MON"
    @retval ElementContainerMatrix
    """
    h=Histogram(runNo=runNo,
                ConvType=ConvType,
                Params=Params,
                DetRange=detectorRange,
                pixelNo=numOfPixel,
                TimeRange=timeRange,
                frameInfo=frameInfo,
                DetType=DetType,
                withPulseHeight=False,
                isDebug=False
                )

    if h.Status==0:
        h.MakeWiringFile()
        return h.GetHistogram()

def GetPulseHeight(runNo=-1,
                   detType="PSD",
                   phHist=[0,10000,10],
                   detectorRange=[-1,-1],
                   timeRange=[-1.0,-1.0]
                   ):
    
    """
    Load EventData and make PulseHeight hist
    GetPulseHeight(runNo=-1,
                   detType="PSD",
                   phHist=[0,10000,10],
                   detectorRange=[-1,-1],
                   timeRange=[-1.0,-1.0]
                   )
    @param runNo
    @param detType
    @param phHist
    @param detectorRange
    @param timeRange
    @retval ElementContainerMatrix
    """
    h=Histogram(runNo=runNo,
                ConvType=2,
                Params=phHist,
                DetRange=detectorRange,
                pixelNo=1,
                TimeRange=timeRange,
                DetType=detType,
                withPulseHeight=True,
                isDebug=False
                )
    
    if h.Status==0:
        h.MakeWiringFile()
        return h.GetHistogram()



# -*- coding: utf-8 -*-

import math
import numpy as np
from numpy import linalg

class ExtMath:
    def  __init__(self):
        pass

    def getElevation(self, x, y, z):
        val = math.sqrt(x*x + y*y)
        ele  = math.atan2(z, val)
        return ele

    def getAzimuth(self, x, y):
        az = math.atan2(y, x)# *180.0/math.pi
        return az

    def getAngle(self, vec1, vec2):
        val = np.dot(vec1, vec2)

        #norm1 = np.linalg.nomr(vec1)  # crash
        norm1 = math.sqrt(vec1[0]*vec1[0]+vec1[1]*vec1[1]+vec1[2]*vec1[2])
        norm2 = math.sqrt(vec2[0]*vec2[0]+vec2[1]*vec2[1]+vec2[2]*vec2[2])

        # domain error ?

        v = val/norm1/norm2

        if v > 1.0:
            v = 1.0
        elif v < -1.0:
            v = -1.0
        #angle = math.acos(v)
        angle = np.arccos(v)

        return angle

    def getRodriguesMatrix(self, vec, rad):
        len = math.sqrt(vec[0]*vec[0]+vec[1]*vec[1]+vec[2]*vec[2])

        if len == 0.0:
            return np.mat(np.eye(3))

        n1 = vec[0]/len
        n2 = vec[1]/len
        n3 = vec[2]/len

        mat = np.mat(np.zeros((3, 3)))

        cosA = math.cos(rad)
        sinA = math.sin(rad)
        mat[0, 0] = cosA + n1*n1*(1-cosA)
        mat[0, 1] = n1*n2*(1-cosA)-n3*sinA
        mat[0, 2] = n1*n2*(1-cosA)+n2*sinA
        mat[1, 0] = n2*n1*(1-cosA)+n3*sinA
        mat[1, 1] = cosA+n2*n2*(1-cosA)
        mat[1, 2] = n2*n3*(1-cosA)-n1*sinA
        mat[2, 0] = n3*n1*(1-cosA)-n2*sinA
        mat[2, 1] = n3*n2*(1-cosA)+n1*sinA
        mat[2, 2] = cosA + n3*n3*(1-cosA)

        return mat

    def rotateAroundVector(self, vec, rad, point):
        mat = self.getRodriguesMatrix(vec, rad)
        newPoint = mat.dot(point)

        return newPoint



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function
from math import pi as PI
from math import sqrt
from scipy import optimize
import numpy as np


class FitOnChoppers(object):
    class Parameter:
        def __init__(self, value):
            self.value = value
            self.init_value = value
            self.value_err = 0.0

        def set(self, value):
            self.value = value

        def __call__(self):
            return self.value

    def fit(self, function, parameters, y, x=None):
        def f(params):
            i = 0
            for p in parameters:
                p.set(params[i])
                i += 1

            return (y - function(x))

        if x is None:
            x = arange(y.shape[0])
        p = [param() for param in parameters]
        result = optimize.leastsq(f, p, full_output=1)

        p = [param() for param in parameters]
        print("-------")
        chisq = sum(f(p)**2)
        print("Chi_sq=", chisq)

        print("-----")
        # [inamura 170306]-->
        # err = sqrt( diag(result[1]) )
        s_sq = chisq / (len(y) - len(parameters))
        err = np.sqrt(np.diag(result[1] * s_sq))
        # <--[inamura 170306]
        i = 0
        for param in parameters:
            param.value_err = err[i]
            print("Param[%2d]  -Initial=%12.8f -Final=%12.8f +-%12.8f" %
                  (i, param.init_value, param(), err[i]))
            i += 1

        print("-------")

        return chisq


'''
Constant value and Unit convert using on Chopper group
'''


class UnitOnChoppers(object):
    '''

    '''

    def __init__(self, filename=None):
        '''
        Constructor
        @param filename (String) New table of Constant values to be imported.
        '''

        self.Const = {}
        self.CUnit = {}

        self.Const['Mass'] = 1.67495e-27
        self.CUnit['Mass'] = "kg"

        self.Const['Planck'] = 6.626068e-34
        self.CUnit['Planck'] = "J*s"

        self.Const['hbar'] = self.Const['Planck'] / (2 * PI)
        self.CUnit['hbar'] = "J*s"

        self.Const['Boltzmann'] = 1.3806503e-23
        self.CUnit['Boltzmann'] = "J/K"

        self.Const['BohrMag'] = 9.27400949e-24
        self.CUnit['BohrMag'] = "J/T"

        if filename is not None:
            self.SetConstTable(filename)

    def ToLamda(self, invalue, inunit):
        '''
        Convert input value to Lambda [Ang] of neutrons
        @param self
        @param invalue (Double) value to be converted
        @param inunit (String) unit of invalue
        @retval tuple (Double,String) Lambda and unit
        '''
        if inunit == "meV":
            ret = 9.045 / sqrt(invalue)
        elif inunit == "1/Ang":
            ret = 6.283 / invalue
        elif inunit == "K":
            ret = 30.81 / sqrt(invalue)
        elif inunit == "Ang":
            ret = invalue
        elif inunit == "m/s":
            ret = (3.956e-3) / invalue
        else:
            print("Invalid Unit {}".format(inunit))
            print("You can use [meV],[1/Ang],[K],[Ang] and [m/s].")
            ret = None

        return (ret, "Ang")

    def ToEnergy(self, invalue, inunit):
        '''
        Convert input value to Energy [meV] of neutrons
        @param self
        @param invalue (Double) value to be converted
        @param inunit (String) unit of invalue
        @retval tuple (Double,String) Energy and unit
        '''

        if inunit == "meV":
            ret = invalue
        elif inunit == "1/Ang":
            ret = 2.072 * (invalue**2)
        elif inunit == "K":
            ret = 0.08617 * invalue
        elif inunit == "Ang":
            ret = 81.81 / (invalue**2)
        elif inunit == "m/s":
            ret = (5.227e-6) * (invalue**2)
        else:
            print("Invalid Unit {}".format(inunit))
            print("You can use [meV],[1/Ang],[K],[Ang] and [m/s].")
            ret = None

        return (ret, "meV")

    def ToVelocity(self, invalue, inunit):
        '''
        Convert input value to velocity [m/s] of neutrons
        @param self
        @param invalue (Double) value to be converted
        @param inunit (String) unit of invalue
        @retval tuple (Double,"m/s") velocity and unit
        '''

        if inunit == "meV":
            ret = (0.4374e+3) * (sqrt(invalue))
        elif inunit == "1/Ang":
            ret = (0.6296e+3) * (invalue)
        elif inunit == "K":
            ret = (0.1284e+3) * (sqrt(invalue))
        elif inunit == "Ang":
            ret = (3.956e+3) / (invalue)
        elif inunit == "m/s":
            ret = invalue
        else:
            print("Invalid Unit {}".format(inunit))
            print("You can use [meV],[1/Ang],[K],[Ang] and [m/s].")
            ret = None

        return (ret, "m/s")



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from __future__ import print_function
import time
import Manyo as mm
import Manyo.Utsusemi as mu
import utsusemi.ana.Reduction.BaseCommands as BaseCom
import os
# special reserved word for commands history
HISTORY = None

# reserved words for return value in this.
DAT = None
RetECA = None
ret = None


#########################################
def CopyContainers(dat=DAT):
    """
    Copy data in SequenceEditer
    @param dat [Def:DAT] (ElementContainer,-Array,-Matrix, List)
    @retval ret (ElementContainer,-Array,-Matrix, List)
    """
    if isinstance(dat, mm.ElementContainer):
        ret = mm.ElementContainer(dat)
    if isinstance(dat, mm.ElementContainerArray):
        ret = mm.ElementContainerArray(dat)
    if isinstance(dat, mm.ElementContainerMatrix):
        ret = mm.ElementContainerMatrix(dat)
    if isinstance(dat, list):
        ret = []
        for item in dat:
            ret.append(item)

    return ret

##########################################


def CalcContainers(dat1=DAT, dat2=DAT, coef1=1.0, coef2=0.0, ope="+", pow1=1.0, pow2=1.0):
    """
    Execute caluculation ret = coef1*dat1 + coef2*dat2
    @param  dat1 [Def:DAT] (ElementContainerMatrix or Array)  first data
    @param  dat2 [Def:DAT] (ElementContainerMatrix or Array)  second data
    @param  coef1  (float)    coefficient of dat1
    @param  coef2  (float)    coefficient of dat2
    @param  ope    (str)      operator
    @param  pow1   (float)    power value for dat1
    @param  pow2   (float)    power value for dat2
    @retval ret    Return Value
    """

    if (coef1 == 0.0) and (coef2 == 0.0):

        raise UserWarning("Both Coefficents are 0.")

    if (coef1 < 0.0) or (coef2 < 0.0):
        raise UserWarning("Coefficents must be positive number.")

    if not (ope in ["+", "-", "*", "/"]):
        raise UserWarning("ope should be +,-,*,/ .")

    if pow1 <= 0.0 or pow2 <= 0.0:
        raise UserWarning("pow1 and pow2 should be over 0.")

    isUsePow = False
    if pow1 != 1.0 or pow2 != 1.0:
        isUsePow = True

    if (coef1 != 0.0) and (coef2 != 0.0):
        if type(dat1) != type(dat2):
            mu.UtsusemiError("Arguments should be same type.")
            raise UserWarning("Arguments should be same type.")

        if isinstance(dat1, mm.ElementContainerMatrix):
            ret = mm.ElementContainerMatrix(dat1.PutHeader())
            tt = mu.UtsusemiCalcContainers()
            if isUsePow:
                tt.CalcContainers(ret, coef1, dat1, pow1, str(ope), coef2, dat2, pow2)
            else:
                tt.CalcContainers(ret, coef1, dat1, str(ope), coef2, dat2)
            del tt
            """
            ret = mm.ElementContainerMatrix(dat1.PutHeader())
            for i in range(dat1.PutTableSize()):
                eca = mm.ElementContainerArray(dat1(i).PutHeader())
                for j in range(dat1(i).PutTableSize()):
                    ec1 = dat1(i,j)
                    ec2 = dat2(i,j)
                    if (ope=="+"):
                        ec = ec1.Mul(coef1)+ec2.Mul(coef2)
                    elif (ope=="-"):
                        ec = ec1.Mul(coef1)-ec2.Mul(coef2)
                    elif (ope=="*"):
                        ec = ec1.Mul(coef1)*ec2.Mul(coef2)
                    else:
                        ec = ec1.Mul(coef1)/ec2.Mul(coef2)

                    ec.InputHeader(dat1(i,j).PutHeader())
                    if ec.PutHeaderPointer().CheckKey("TotalCounts")==1:
                        ec.PutHeaderPointer().OverWrite("TotalCounts",sum(ec.PutYList()))
                    else:
                        ec.AddToHeader("TotalCounts",sum(ec.PutYList()))
                    eca.Add(ec)
                ret.Add(eca)
            """
        if isinstance(dat1, mm.ElementContainerArray):
            ret = mm.ElementContainerArray(dat1.PutHeader())
            tt = mu.UtsusemiCalcContainers()
            if isUsePow:
                tt.CalcContainers(ret, coef1, dat1, pow1, str(ope), coef2, dat2, pow2)
            else:
                tt.CalcContainers(ret, coef1, dat1, str(ope), coef2, dat2)
            del tt
            """
            for i in range(dat1.PutTableSize()):
                ec1=dat1(i)
                ec2=dat2(i)
                if (ope=="+"):
                    ec = ec1.Mul(coef1)+ec2.Mul(coef2)
                elif (ope=="-"):
                    ec = ec1.Mul(coef1)-ec2.Mul(coef2)
                elif (ope=="*"):
                    ec = ec1.Mul(coef1)*ec2.Mul(coef2)
                else:
                    ec = ec1.Mul(coef1)/ec2.Mul(coef2)

                ec.InputHeader(dat1(i).PutHeader())
                if ec.PutHeaderPointer().CheckKey("TotalCounts")==1:
                    ec.PutHeaderPointer().OverWrite("TotalCounts",sum(ec.PutYList()))
                else:
                    ec.AddToHeader("TotalCounts",sum(ec.PutYList()))
                ret.Add(ec)
            """
        if isinstance(dat1, mm.ElementContainer):
            if isUsePow:
                d1 = dat1.Pow(pow1)
                d2 = dat2.Pow(pow2)
            else:
                d1 = dat1
                d2 = dat2
            ec = None
            if (ope == "+"):
                ec = d1.Mul(coef1) + d2.Mul(coef2)
            elif (ope == "-"):
                ec = d1.Mul(coef1) - d2.Mul(coef2)
            elif (ope == "*"):
                ec = d1.Mul(coef1) * d2.Mul(coef2)
            else:
                ec = d1.Mul(coef1) / d2.Mul(coef2)

            return ec

        if isinstance(dat1, list):
            if len(dat1) == 0:
                raise UserWarning("Argument1 is empty list")
            if not isinstance(dat1[0], mm.ElementContainer):
                raise UserWarning("Argument1 has no ElementContainer")
            if len(dat1) != len(dat2):
                raise UserWarning(
                    "The number of ElmentContainer in Argument1 and Argument2 are different")

            ret = []
            for ec1o, ec2o in zip(dat1, dat2):
                if isinstance(ec1o, mm.ElementContainer) and isinstance(ec2o, mm.ElementContainer):
                    if isUsePow:
                        ec1 = ec1o.Pow(pow1)
                        ec2 = ec2o.Pow(pow2)
                    else:
                        ec1 = ec1o
                        ec2 = ec2o
                    if (ope == "+"):
                        ec = ec1.Mul(coef1) + ec2.Mul(coef2)
                    elif (ope == "-"):
                        ec = ec1.Mul(coef1) - ec2.Mul(coef2)
                    elif (ope == "*"):
                        ec = ec1.Mul(coef1) * ec2.Mul(coef2)
                    else:
                        ec = ec1.Mul(coef1) / ec2.Mul(coef2)

                    ret.append(ec)
                else:
                    try:
                        if (ope == "+"):
                            ec = ec1 * coef1 + ec2 * coef2
                        elif (ope == "-"):
                            ec = ec1 * coef1 - ec2 * coef2
                        elif (ope == "*"):
                            ec = ec1 * coef1 * ec2 * coef2
                        else:
                            ec = (ec1 * coef1) / (ec2 * coef2)
                    except:
                        raise UserWarning("Arguments are NOT calculable")

                    ret.append(ec)
        return ret

    if (coef1 == 0):
        if isinstance(dat2, list):
            ret = []
            for ec2 in dat2:
                if isinstance(ec2, mm.ElementContainer):
                    ec2_temp = ec2.Pow(pow2)
                    ret.append(ec2_temp.Mul(coef2))
        else:
            dat2_temp = dat2.Pow(pow2)
            ret = dat2_temp.Mul(coef2)
        return ret

    if (coef2 == 0):
        if isinstance(dat1, list):
            ret = []
            for ec1 in dat1:
                if isinstance(ec1, mm.ElementContainer):
                    ec1_temp = ec1.Pow(pow1)
                    ret.append(ec1_temp.Mul(coef1))
        else:
            dat1_temp = dat1.Pow(pow1)
            ret = dat1_temp.Mul(coef1)
        return ret


#########################################
def SearchHeaderInteger(Target=DAT, HeaderKey="PIXELID", Min=0, Max=0):
    """
    Search in each ElementContainer with Header with HeaderKey and its Value from Matrix.
    @param  Target [Def:DAT]     ElementContainerMatrix
    @param  HeaderKey  string Key of Integer Header for search
    @param  Min        minimun value for search
    @param  Max        maximum value for search
    @retval RetECA     search results in ElementContainerArray
    """

    if isinstance(Target, mm.ElementContainerMatrix):
        ECM = Target
    else:
        mu.UtsusemiError("This Target is NOT suitable argument!")
        return None

    if ((HeaderKey == "") and (Max == "")):
        return None

    sh = mm.SearchInHeader(ECM)

    if (Min == Max):
        sh.Search(HeaderKey, Min)
    else:
        sh.Search(HeaderKey, Min, Max)

    RetECA = sh.PutResultAsArray()
    if (RetECA.PutTableSize() == 0):
        mu.UtsusemiWarning("No data corresponding with given parameter")

    del sh
    return RetECA

#########################################


def SearchHeaderDouble(Target=DAT, HeaderKey="", Min=0.0, Max=0.0):
    """
    Search in each ElementContainer with Header with HeaderKey and its Value from Matrix.
    @param  Target [Def:DAT]     ElementContainerMatrix
    @param  HeaderKey  string Key of Double Header for search
    @param  Min        minimun value for search
    @param  Max        maximum value for search
    @retval RetECA     search results in ElementContainerArray
    """

    if isinstance(Target, mm.ElementContainerMatrix):
        ECM = Target
    else:
        mu.UtsusemiError("This Target is NOT suitable argument!")
        return None

    if ((HeaderKey == "") and (Max == "")):
        return None

    sh = mm.SearchInHeaders()
    sh.SetTarget(ECM)
    if (Min == Max):
        sh.Search(HeaderKey, Min)
    else:
        sh.Search(HeaderKey, Min, Max)

    RetECA = sh.PutCurrent()
    if (RetECA.PutTableSize() == 0):
        mu.UtsusemiWarning("No data corresponding with given parameter")

    return RetECA


#########################################
def ExecCode(Code="A.Dump()", A=DAT):
    """
    Execute given Code : Input code "print 1" means to execute "print 1" at console
    @param  Code     string  Equation using A
    @param  A [Def:DAT] 1st Variable (ElementContainer, some variables)
    """

    exec(Code)
    return


#########################################
def ExecFunctionCode(Code="A+B", A=DAT, B=DAT):
    """
    Execute given Code : Input code "A+B" means to execute "RET=A+B"
    @param  Code     string  Equation using A and B
    @param  A [Def:DAT] 1st Variable (ElementContainer, some variables)
    @param  B [Def:DAT] 2nd Variable (ElementContainer, some variables)
    @retval ret      Return Value
    """

    # exe_code = "ret="+Code
    # exec(exe_code)
    ret = eval(Code)
    return ret


#########################################
def ExecCodeManyArgs(Code="A.Dump()", A=DAT, B=DAT, C=DAT, D=DAT, E=DAT, F=DAT):
    """
    Execute given Code : Input code "print 1" means to execute "print 1" at console
    @param  Code        string  Equation using A
    @param  A [Def:DAT]  1st Variable (ElementContainer, some variables)
    @param  B [Def:EMPTYDAT]  2nd Variable (ElementContainer, some variables)
    @param  C [Def:EMPTYDAT]  3rd Variable (ElementContainer, some variables)
    @param  D [Def:EMPTYDAT]  4th Variable (ElementContainer, some variables)
    @param  E [Def:EMPTYDAT]  5th Variable (ElementContainer, some variables)
    @param  F [Def:EMPTYDAT]  6th Variable (ElementContainer, some variables)
    """

    exec(Code)
    return None

#########################################


def ExecFunctionCodeManyArgs(Code="A+B", A=DAT, B=DAT, C=DAT, D=DAT, E=DAT, F=DAT):
    """
    Execute given Code : Input code "A+B" means to execute "RET=A+B"
    @param  Code        string  Equation using A and B
    @param  A [Def:DAT]  1st Variable (ElementContainer, some variables)
    @param  B [Def:EMPTYDAT]  2nd Variable (ElementContainer, some variables)
    @param  C [Def:EMPTYDAT]  3rd Variable (ElementContainer, some variables)
    @param  D [Def:EMPTYDAT]  4th Variable (ElementContainer, some variables)
    @param  E [Def:EMPTYDAT]  5th Variable (ElementContainer, some variables)
    @param  F [Def:EMPTYDAT]  6th Variable (ElementContainer, some variables)
    @retval ret      Return Value
    """

    # exe_code = "ret="+Code
    # exec(exe_code)
    ret = eval(Code)
    return ret


def MakeECList(DAT=DAT, Min=0, Max=0):
    """
    Make list of ElementContainers from ElementContainerArray
    @param DAT [Def:DAT] ElementContainerArray
    @param Min
    @param Max
    @retval ecs (list of ElementContainers)
    """
    ret = []
    if isinstance(DAT, mm.ElementContainerArray):
        if Min == 0 and Max == 0:
            for i in range(DAT.PutSize()):
                ret.append(DAT.Put(i))
        else:
            for i in range(Min, (Max + 1)):
                ret.append(DAT.Put(i))
    return ret

#########################################


def ArrayDataCut(Dat=DAT, Axis="X", Xkey="Default", IntegRange="All", Ave=True, AxisRange="All"):
    """
    Cut the given data (ElementContainerArray)
    If given data is ElementContainerMatrix, first ElementContainerArray is used for cutting
    @param  Dat [Def:DAT]   data
    @param  AxisX (string) axis of cut data. can use a key of vector stored in header of ElementContainer
    @param  IntegRange (string) range of integral on data "All" or "<min>,<max>"
    @param  Ave (bool) True : cut intensity is averaging, False : summation.
    @param  AxisRange (string) : range of axis "All" or "<min>,<max>"
    @retval EC (ElementContaienr)
    """
    BaseCom.ArrayDataCut(Dat, Axis, Xkey, IntegRange, Ave, AxisRange)

#########################################


def IntegPixels(Target=DAT, PSD_min=1, PSD_max=1, Pixel_min=1, Pixel_max=1, isAve=True):
    """
    Put summation of intensity along Pixel
    @param Target [Def:DAT] (ElementContainerMatrix)
    @param PSD_min   (int) lower limit of PSD-ID range
    @param PSD_max   (int) upper limit of PSD-ID range
    @param Pixel_min (int) lower limit of Pixel range
    @param Pixel_max (int) upper limit of Pixel range
    @param isAve     (bool) averaging or summation
    @retval EC ElementContainer
    """
    return BaseCom.IntegPixels(Target, PSD_min, PSD_max, Pixel_min, Pixel_max, isAve)


# Dictionary for entry the name of functions
_functions = {
    "CopyContainers": CopyContainers,
    "CalcContainers": CalcContainers,
    "IntegPixels": IntegPixels,
    "ArrayDataCut": ArrayDataCut,
    "SearchHeaderInteger": SearchHeaderInteger,
    "SearchHeaderDouble": SearchHeaderDouble,
    "ExecCode": ExecCode,
    "ExecFunctionCode": ExecFunctionCode,
    "ExecCodeManyArgs": ExecCodeManyArgs,
    "ExecFunctionCodeManyArgs": ExecFunctionCodeManyArgs,
    "MakeECList": MakeECList
}

_functionsOrder = ["CopyContainers",
                   "CalcContainers",
                   "IntegPixels",
                   "ArrayDataCut",
                   "SearchHeaderInteger",
                   "SearchHeaderDouble",
                   "ExecCode",
                   "ExecFunctionCode",
                   "ExecCodeManyArgs",
                   "ExecFunctionCodeManyArgs", "MakeECList"]



import os
import sys
import datetime

ver = float(sys.version_info[0]) + float(sys.version_info[1]) * 0.1
if ver < 2.5:
    from elementtree.ElementTree import ElementTree, fromstring, XML, tostring
else:
    from xml.etree.ElementTree import ElementTree, fromstring, XML, tostring

"""
Installer of UTSUSEMI
To install UTSUSEMI environment into Ubuntu 12.04, 14.04 and 16.04 LTS with Manyo release version
VerC0.3.7-1  : Enabled for Ubuntu 18.04
VerC0.3.7    : DNA disabled for 0.3.7 (under construction)
VerC0.3.6    : DNA Enabled
               Note : Ubuntu packages 'libblas-dev' and 'liblapack-dev' required

VerC0.3.3.3  : bugfix for Default Installation
verC0.3.3.2  : bugfix for SAS (171227)
verC0.3.3.1  : for pyqtgraph
verC0.3.3.0-2: for Ubuntu 16.04LTS
verC0.3.3.0-1: bugfix the issue when overwrite update
verC0.3.3.0 : for release 0.3.3.0
"""
__version__ = "C0.3.7-1"

############################################################


class InstallToUbuntu:
    ########################################################
    def __init__(self, debug=False):
        """
        Constructor
        @param debug (bool) if debug=True, installer does not touch disk
        @return None
        """
        self.debug = debug
        self.logs = []
        self.logs_index = 0
        self.DefInstBL = 1
        self.cdir = os.getcwd()
        self.MLFDEVROOT = os.path.join(os.environ["HOME"], ".MLFENV")
        if not os.path.exists(self.MLFDEVROOT):
            os.system("mkdir %s" % self.MLFDEVROOT)
        self.MLFDEVENVFILE = os.path.join(self.MLFDEVROOT, "environ.xml")

        self.PathDic = {}
        self.PathDic["mlfsoft_root"] = ""  # for mlfsoft
        self.PathDic["data_root"] = ""     # for /data/SIK or /data/AMR
        self.PathDic["userhome"] = ""      # for ~/ana/ and ~/exp/
        self.PathDic["mlfenv"] = ""        #
        self.PathDic["environfile"] = ""

        self.MakeDic = {}
        self.MakeDic["CXX"] = ""
        self.MakeDic["PYVER"] = ""
        self.MakeDic["NUM_OF_THREADS"] = ""

        self.InstallList = []

        self._initialEnviron()

        self.dom = None

        self.date_fomat = "[%4d/%2d/%2d %2d:%2d:%2d] "
        # self._checkfiles()

    ########################################################
    def _initialEnviron(self):
        self.PathDic["mlfsoft_root"] = "/opt/mlfsoft"  # for mlfsoft
        self.PathDic["data_root"] = "/data"            # for /data/SIK or /data/AMR
        self.PathDic["userhome"] = os.environ["HOME"]  # for ~/ana/ and ~/exp/

        self.MakeDic["CXX"] = "g++"
        self.MakeDic["PYVER"] = "2.7"
        self.MakeDic["NUM_OF_THREADS"] = 8

    ########################################################
    def _readEnviron(self):
        """
        """
        if not os.path.exists(self.MLFDEVENVFILE):
            msg = " There is no saved environment file (%s)." % self.MLFDEVENVFILE
            print msg
            self._writeLog(msg)
            self._saveEnviron()
        else:
            fd = file(self.MLFDEVENVFILE, 'r')
            dom = ElementTree(file=fd)
            fd.close()

            root_ele = dom.getroot()

            try:
                self.PathDic["mlfsoft_root"] = root_ele.find("mlfSoftRoot").text
                self.PathDic["data_root"] = root_ele.find("mlfBaseDataPath").text
                self.PathDic["userhome"] = root_ele.find("mlfUserHome").text
                self.DefInstBL = int(root_ele.find("mlfInstBL").text)
            except:
                msg = "Invalid parameter file."
                print msg
                self._writeLog(msg)

        # self._showEnviron()

    ########################################################
    def _showEnviron(self):

        print "----------------------------------------------------------"
        print " Install Directories"
        print "    MLF software root (Manyo, Utsusemi): %s" % self.PathDic["mlfsoft_root"]
        print "    User's home directory              : %s" % self.PathDic["userhome"]
        print "    RAW data  Directory                : %s" % self.PathDic["data_root"]
        print "----------------------------------------------------------"

    ########################################################

    def _saveEnviron(self):
        """
        """
        def s_tab(x):
            kk = ""
            for i in range(x):
                kk += "    "
            return kk

        ss = "<utsusemiOnUbuntuEnviron>\n"
        ss += s_tab(1) + ("<mlfSoftRoot>%s</mlfSoftRoot>\n" % (self.PathDic["mlfsoft_root"]))
        ss += s_tab(1) + ("<mlfBaseDataPath>%s</mlfBaseDataPath>\n" % (self.PathDic["data_root"]))
        ss += s_tab(1) + ("<mlfUserHome>%s</mlfUserHome>\n" % (self.PathDic["userhome"]))
        ss += s_tab(1) + ("<mlfInstBL>%d</mlfInstBL>\n" % (self.DefInstBL))
        ss += "</utsusemiOnUbuntuEnviron>\n"

        xml = XML(ss)
        try:
            fd = open(self.MLFDEVENVFILE, "w")
            fd.write('<?xml version="1.0" encoding="utf-8"?>\n')
            fd.write(tostring(xml))
            fd.close()
        except:
            msg = "Cannot open %s for saving instalation parameters." % self.MLFDEVENVFILE
            print msg
            self._writeLog(msg)

    ########################################################
    def _checkfiles(self, foldername):
        """
        Check directory or file name required for install
        @param  foldername
        @return None
        """
        self.cdir = os.getcwd()
        src_dir = os.path.join(self.cdir, "src")
        dlist = os.listdir(src_dir)

        isExist = False
        for item in dlist:
            if (item.find(str(foldername)) != -1) and (os.path.isdir(os.path.join(src_dir, item))):
                isExist = True
                break
            else:
                msg = "There is no source directory for %s." % (foldername)
                isExist = False

        return isExist

    ########################################################
    def uninstall(self, delDataFlag=False):
        """
        Uninstall procedure
        @param delDataFlag (bool) if True, delete Raw data and xml files
        @retval 0  No problem
        @retval -1 Error happened
        """

        self._readEnviron()

        # InstCode = self.DicInstCode[ self.DefInstBL ]
        # InstDir = os.path.join(self.IROHAroot,('mlf_%s' % InstCode ) )
        coms = []
        coms.append("sudo rm -rf %s" % (self.PathDic["mlfsoft_root"]))
        msg = 'self.PathDic["mlfsoft_root"]= %s' % self.PathDic["mlfsoft_root"]
        self._writeLog(msg)

        coms.append("sudo rm -f /etc/ld.so.conf.d/manyo.conf")

        if delDataFlag:
            coms.append("rm -rf %s" % os.path.join(self.PathDic["userhome"], "ana"))
            coms.append("rm -rf %s" % os.path.join(self.PathDic["userhome"], "exp"))
            coms.append("sudo rm -rf %s" % self.PathDic["data_root"])

        flist = os.listdir(self.cdir)
        for item in flist:
            if (item.find("manyo") != -1) and (os.path.isdir(item)):
                coms.append("rm -rf manyo*")
            if (item.find("python") != -1) and (os.path.isdir(item)):
                coms.append("rm -rf python-*")
            if (item.find("do_make_Utsusemi.py") != -1):
                coms.append("rm -rf do_make*.py")

        try:
            for com in coms:
                os.system(com)
                self._writeLog(com)
        except:
            return -1

        return 0

    ########################################################

    def AskPrivateInstallPath(self):
        """

        @param None
        @retval  0    OK
        @retval -1    do nothing
        """

        self.PathDic["mlfsoft_root"] = os.path.join(os.environ["HOME"], "mlfsoft")
        self.PathDic["data_root"] = os.path.join(os.environ["HOME"], "mlfsoft", "RAWDATA")
        self.PathDic["userhome"] = os.environ["HOME"]

        print "============================================================"
        print "                   Set your install paths"
        while(True):
            print " "
            print "============================================================"
            print " 1. Path to install Utsusemi software"
            print "   --> %s" % self.PathDic["mlfsoft_root"]
            print " "
            print " 2. Path to Event data directory"
            print "   --> %s" % self.PathDic["data_root"]
            print " "
            print " 3. Path to user directory (for ana/xml or exp/xml)"
            print "   --> %s" % self.PathDic["userhome"]
            print "============================================================"
            print " "
            ret = raw_input(">> Which do you change ? [1 to 3. 0 to exit] ")
            if ret == "1":
                r_dir = self._AskNewPath("install Utsusemi software")
                if r_dir != "":
                    self.PathDic["mlfsoft_root"] = r_dir
                continue
            elif ret == "2":
                r_dir = self._AskNewPath("Event data directory")
                if r_dir != "":
                    self.PathDic["data_root"] = r_dir
                continue
            elif ret == "3":
                r_dir = self._AskNewPath("User directory")
                if r_dir != "":
                    self.PathDic["userhome"] = r_dir
                continue
            elif ret == "0":
                break
            else:
                print "Input value is invalid."

        return 0
    ########################################################

    def _AskNewPath(self, title):
        """
        """
        print "Please imput new path to %s = " % title
        path_in = raw_input(">> ")
        if path_in == "":
            return ""

        coms = []
        coms.append("mkdir -p %s" % (path_in))
        if -1 == self._doCommands(coms, self.debug):
            return ""

        return path_in
        """
        ret = self._makeNewDir( path_in )
        if ret==-1:
            return ""

        return ret
        """

    ########################################################
    def installUbuntuPackages(self):
        """
        Install Ubuntu Package to be required for installation
        @retval 0  No problem
        @retval -1 Error happend
        """
        coms = []

        pkg_list = ['g++', 'subversion', 'swig',
                    'python-dev', 'python-wxgtk2.8', 'python-vtk',
                    'python-numpy', 'python-scipy', 'python-matplotlib', 'python-h5py',
                    'libhdf5-serial-dev', 'libmxml-dev', 'libgsl0-dev', 'libbz2-dev',
                    'libnexus0-dev', 'libblas-dev', 'liblapack-dev'
                    ]

        coms.append("sudo apt-get update")
        coms.append("sudo apt-get upgrade")
        self._doCommands(coms, self.debug, True)

        # [inamura 140828]-->
        # search and add packages for boost
        import commands
        tmp = commands.getoutput("cat /etc/issue")
        tmp_l = tmp.split(" ")
        ver_utsu = tmp_l[1]
        if ver_utsu[:2] == "12":
            tmp = commands.getoutput("sudo apt-cache search libboost1")
            tmp_lines = tmp.split("\n")
            libboost_list = []
            for i in tmp_lines:
                tmp_list = i.split(" - ")
                if tmp_list[0].find("-all-dev") != -1:
                    libboost_list.append(tmp_list[0])
            libboost_list.sort()
            pkg_list.append(libboost_list[-1])
        elif ver_utsu[:2] == "14":
            pkg_list.append()
            pkg_list.append("libboost-all-dev")
        elif ver_utsu[:2] == "16":
            pkg_list.append("libgsl-dev")
            pkg_list.append("libboost-all-dev")
            pkg_list.remove('python-wxgtk2.8')
            pkg_list.append('python-wxgtk3.0')
            pkg_list.append('python-pip')
            pkg_list.append('python-qt4-gl')
            pkg_list.append('python-qt4')
            pkg_list.append('python-opengl')
        elif ver_utsu[:2] == "18":
            pkg_list.remove('python-vtk')
            pkg_list.remove('python-wxgtk2.8')
            pkg_list.append('python-wxgtk3.0')
            pkg_list.append("python-vtk6")
            pkg_list.append("libgsl-dev")
            pkg_list.append("libboost-all-dev")
            pkg_list.append('python-wxgtk3.0')
            pkg_list.append('python-pip')
            pkg_list.append('python-qt4-gl')
            pkg_list.append('python-qt4')
            pkg_list.append('python-opengl')
        else:
            msg = "Ubuntsu Linux version is not suitable."
            raise UserWarning(msg)
        # -->[inamura 140828]

        coms = []
        if ver_utsu[:2] == "18":
            for pkg in pkg_list:
                coms.append(('sudo apt install -y %s' % pkg))
        else:
            for pkg in pkg_list:
                coms.append(('sudo apt-get install -y %s' % pkg))

        coms.append('pip install pyqtgraph')

        return self._doCommands(coms, self.debug)

    ########################################################
    def _doCommands(self, coms, debugFlag=False, ignoreErr=False):
        """
        Execute command list
        @param coms (list of string) commands to be executed
        @param debugFlag (bool) if True, do not touch files on disk
        @retval 0 No problem
        @retval -1 Error happened
        """
        if not isinstance(debugFlag, bool):
            msg = "doCommands: cannot invalid flag"
            print msg
            self._writeLog(msg)
            return -1

        for com in coms:
            com_sudo = ""
            if not debugFlag:

                ff = os.system(com)
                msg = "_doCommands::%s res=%d" % (com, ff)
                self._writeLog(msg)
                if ff != 0:
                    msg = "_doCommands::sudo %s" % com
                    self._writeLog(msg)
                    com_sudo = 'sudo ' + com
                    if os.system(com_sudo) != 0:
                        if not ignoreErr:
                            msg = "sudo fails: %s" % com_sudo
                            self._writeLog(msg)
                            return -1

            if com_sudo != "":
                self.logs.append(com_sudo)
            else:
                self.logs.append(com)

        return 0
    ########################################################

    def _writeLogs(self):
        """
        Save logs to file

        """
        fo = open("install_log.txt", 'a')
        fo.write("                            \n")
        fo.write("------- Write Logs ---------\n")
        for log_line in self.logs[self.logs_index:]:
            dt = datetime.datetime.today()
            out_log = (self.date_fomat % (dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second)) + log_line + "\n"
            fo.write(out_log)
        fo.write("----------------------------\n")
        fo.close()
        self.logs_index = len(self.logs)

    ########################################################
    def _writeLog(self, message):
        """
        """
        fo = open("install_log.txt", "a")
        dt = datetime.datetime.today()
        df = (self.date_fomat % (dt.year, dt.month, dt.day, dt.hour, dt.minute, dt.second))
        fo.write(df + "Message >> " + message + "\n")
        fo.close()

    ########################################################
    def Check_Scenario(self, scenario, pre_install_pack=[]):
        """
        Scenario item = ( <source name>,[<required source name>,<...>],<source type> )
        <source name> : directory name like 'manyo-core', 'python-utsusemi'
        <source type> : 'manyo' or 'python' or 'python-utsusemi'
                        if manyo -> compile source
                        if python -> copy directory to /opt/mlfsoft/<source name>
                        if python-utsusemi -> copy direcotry to /opt/mlfsoft/python-utsusemi/
        """
        dicSe = pre_install_pack[:]
        self.InstallList = []
        for pname, depps, src_type in scenario:
            if pname in dicSe:
                pass
            else:
                dicSe.append(pname)
            for deppack in depps:
                if deppack in dicSe:
                    pass
                else:
                    msg = "Check_Scenario >> Senario is invalid. %s must be installed before %s." % (deppack, pname)
                    print msg
                    self._writeLog(msg)
                    return False
            if self._checkfiles(pname):
                self.InstallList.append((pname, src_type))
            else:
                msg = "Check_Scenario >> checkfiles is invalid : No source folder for (%s)." % (pname)
                print msg
                self._writeLog(msg)
                return False

        return True

    ########################################################
    def Install_as_Scenario(self, scenario=[], pre_installed_pack=[]):
        """

        """
        if len(scenario) != 0:
            if not self.Check_Scenario(scenario, pre_installed_pack):
                return False

        elif len(self.InstallList) == 0:
            return False

        self.InstallList.reverse()
        for dname, a_type in self.InstallList:
            if a_type == "libs":
                if dname == "lib-levmar":
                    if self.Install_Levmar(dname):
                        return False

        for dname, a_type in self.InstallList:
            if a_type == "manyo":
                do_make_file_path = os.path.join(self.cdir, "src", dname, "do_make_Utsusemi.py")
                if os.path.exists(do_make_file_path):
                    ro = open(do_make_file_path, "r")
                    do_make_coms = ro.readlines()
                    ro.close()
                    out_make_coms = []
                    for a_com in do_make_coms:
                        if a_com.find("CXX") == 0:
                            out_make_coms.append('CXX            = "%s"\n' % (self.MakeDic["CXX"]))
                        elif a_com.find("PYVER") == 0:
                            out_make_coms.append('PYVER          = "%s"\n' % (self.MakeDic["PYVER"]))
                        elif a_com.find("NUM_OF_THREADS") == 0:
                            out_make_coms.append('NUM_OF_THREADS = %d\n' % (self.MakeDic["NUM_OF_THREADS"]))
                        elif a_com.find("MANYO_PATH") == 0:
                            out_make_coms.append('MANYO_PATH     = "%s"\n' % (os.path.join(self.PathDic["mlfsoft_root"], "manyo")))
                        elif a_com.find("isCompPara") == 0:
                            out_make_coms.append("isCompPara = False\n")
                        else:
                            out_make_coms.append(a_com)
                    rw = open(os.path.join(self.cdir, "src", "do_make_Utsusemi.py"), "w")
                    if rw is None:
                        return False
                    rw.writelines(out_make_coms)
                    rw.close()
        if os.path.exists(os.path.join(self.cdir, "src", "do_make_Utsusemi.py")):
            pass
        else:
            return False

        self.InstallList.reverse()
        coms = []
        for dname, a_type in self.InstallList:
            if a_type == "manyo":
                coms.append("mkdir -p %s" % (self.PathDic["mlfsoft_root"]))
                coms.append("chmod 777 -R %s" % (self.PathDic["mlfsoft_root"]))
                coms.append("cd %s; python ../do_make_Utsusemi.py install" % (os.path.join(self.cdir, "src", dname)))
            elif a_type == "python":
                p1 = os.path.join(self.cdir, "src", dname)
                p2 = os.path.join(self.PathDic["mlfsoft_root"], dname)
                coms.append("mkdir -p %s" % (p2))
                coms.append("cp -rf %s/* %s" % (p1, p2))
            elif a_type == "python-utsusemi":
                p1 = os.path.join(self.cdir, "src", dname)
                fname = dname[(len(a_type) + 1):]
                p2 = os.path.join(self.PathDic["mlfsoft_root"], a_type, fname)
                coms.append("mkdir -p %s" % (p2))
                coms.append("cp -rf %s/* %s" % (p1, p2))
            elif a_type == "env":
                p1 = os.path.join(self.cdir, "src", dname)
                fname = dname[(len(a_type) + 1):]
                coms.append("cp -rf %s/* %s" % (p1, self.PathDic["userhome"]))

        if self._doCommands(coms) < 0:
            return False

        print " "
        print "-----------------------------------------------"
        print "  Install of "
        for a_pack in self.InstallList:
            print "       - %s " % (a_pack[0])
        print "                                were completed."
        print "-----------------------------------------------"
        print " "

        return True

    ########################################################
    def Install_Levmar(self, dname):
        levmar_path = os.path.join(self.cdir, "src", dname)
        manyo_lib_path = os.path.join(self.PathDic["mlfsoft_root"], "manyo", "lib")
        manyo_inc_path = os.path.join(self.PathDic["mlfsoft_root"], "manyo", "include")
        coms = []
        coms.append("cd %s; make" % (levmar_path))
        coms.append("sudo mkdir -p %s" % (self.PathDic["mlfsoft_root"]))
        coms.append("sudo chmod 777 -R %s" % (self.PathDic["mlfsoft_root"]))
        coms.append("mkdir -p %s" % (manyo_lib_path))
        coms.append("mkdir -p %s" % (manyo_inc_path))
        coms.append("cp -rf %s/liblevmar.* %s " % (levmar_path, manyo_lib_path))
        coms.append("cp -rf %s/*.h %s " % (levmar_path, manyo_inc_path))

        if self._doCommands(coms) < 0:
            print "Installation of Levmar Library failed."
            return False

    ########################################################
    def make_Utsusemi_Env(self):
        """

        """

        user_dirs = []
        user_dirs.append(os.path.join(self.PathDic["userhome"], "ana"))
        user_dirs.append(os.path.join(self.PathDic["userhome"], "ana", "tmp"))
        user_dirs.append(os.path.join(self.PathDic["userhome"], "ana", "xml"))
        user_dirs.append(os.path.join(self.PathDic["userhome"], "exp"))
        user_dirs.append(os.path.join(self.PathDic["userhome"], "exp", "log"))
        user_dirs.append(os.path.join(self.PathDic["userhome"], "exp", "xml"))

        coms = []
        for udir in user_dirs:
            if not os.path.exists(udir):
                coms.append('mkdir ' + udir)

        coms.append("mkdir -p %s" % (self.PathDic["mlfsoft_root"]))
        coms.append("sudo chmod 777 -R %s" % (self.PathDic["mlfsoft_root"]))
        coms.append('mkdir -p %s' % (self.PathDic["data_root"]))
        coms.append('sudo chmod 777 -R %s' % (self.PathDic["data_root"]))

        ret = self._doCommands(coms)
        if ret == -1:
            msg = "Error happends on executing command %s." % self.logs[-1]
            print msg
            self._writeLog(msg)
            self._writeLogs()
            raise "ERROR", msg

        self._writeLogs()

        self._saveEnviron()

        print " "
        print "-------------------------------------------------------"
        print "  Utsusemi Base Environment Setting has been completed."
        print "-------------------------------------------------------"
        print " "

    ########################################################
    def add_EnvironmaneVariables(self, BLcode):
        """
        """
        bash_file = os.path.join(self.PathDic["mlfsoft_root"], "python-utsusemi", BLcode, "bashrc.%3s" % (BLcode))
        if not os.path.exists(bash_file):
            msg = "Error: There is no bashrc template file (%s)\n" % (bash_file)
            print msg
            self._writeLog(msg)
            raise UserWarning(msg)

        fo = open(bash_file, "r")
        bash_conts = fo.readlines()
        fo.close()

        rewrite_lines = []
        for a_line in bash_conts:
            if a_line[:3] == "XXX":
                rewrite_lines.append("XXX=%s\n" % (BLcode))
            elif a_line[:2] == "BL":
                pass
            elif a_line[:17] == "export MANYO_PATH":
                rewrite_lines.append("export MANYO_PATH=%s\n" % (os.path.join(self.PathDic["mlfsoft_root"], "manyo")))
            elif a_line[:15] == "UTSUSEMIPY_PATH":
                rewrite_lines.append("UTSUSEMIPY_PATH=%s\n" % (os.path.join(self.PathDic["mlfsoft_root"], "python-utsusemi")))
            elif a_line[:15] == "PYUTSUSEMI_PATH":
                rewrite_lines.append("PYUTSUSEMI_PATH=%s\n" % (os.path.join(self.PathDic["mlfsoft_root"], "python-utsusemi")))
            elif a_line[:25] == "UTSUSEMI_EXT_LIBRARY_PATH":
                rewrite_lines.append("UTSUSEMI_EXT_LIBRARY_PATH=%s\n" % (self.PathDic["mlfsoft_root"]))
            else:
                rewrite_lines.append(a_line)

        fw = open(bash_file, "w")
        fw.writelines(rewrite_lines)
        fw.close()

        # add bashrc
        ss = '\n'
        ss += '# For Utsusemi environment\n'
        ss += 'if [ -f %s ]; then\n' % (bash_file)
        # ss+= '       source %s\n' %(bash_file)
        ss += '       alias %s="source %s"\n' % (BLcode, bash_file)
        ss += 'fi\n'

        try:
            fw = open("add.bashrc", "w")
        except:
            msg = "Cannot open add.bashrc."
            print msg
            self._writeLog(msg)
            return -1
        fw.write(ss)
        fw.close()

        coms = []
        coms.append('cat add.bashrc >> ~/.bashrc')
        coms.append('rm -rf add.bashrc')
        coms.append('mkdir -p %s' % (os.path.join(self.PathDic["data_root"], BLcode)))
        coms.append('sudo chmod 777 -R %s' % (self.PathDic["data_root"]))

        if (BLcode == "DNA"):
            coms.append("cp -rf src/environ-DNA/ana/* %s" % (os.path.join(self.PathDic["userhome"], "ana")))

        ret = self._doCommands(coms, self.debug)
        if ret == -1:
            return -1

        # End safely
        print " "
        print "----------------------------------------------------------"
        print "  Utsusemi Environment Setting for %3s has been completed." % (BLcode)
        print "----------------------------------------------------------"
        print " "
        return 0


PACK_DIC = {"MANYO-CORE": ("manyo-core", [], "manyo"),
            "MANYO-UTSU": ("manyo-utsusemi", ["manyo-core"], "manyo"),
            "PYTHON-UTSU": ("python-utsusemi", ["manyo-utsusemi"], "python"),
            "PYTHON-UTSU-SIK": ("python-utsusemi-SIK", ["python-utsusemi"], "python-utsusemi"),
            "PYTHON-UTSU-AMR": ("python-utsusemi-AMR", ["python-utsusemi"], "python-utsusemi"),
            "PYTHON-UTSU-HPN": ("python-utsusemi-HPN", ["python-utsusemi"], "python-utsusemi"),
            "MANYO-TKT": ("manyo-TKtools", ["manyo-core"], "manyo"),
            "PYTHON-UTSU-TKT": ("python-utsusemi-TKtools", ["manyo-TKtools", "python-utsusemi"], "python-utsusemi"),
            "MANYO-SAS": ("manyo-SAS", ["manyo-utsusemi"], "manyo"),
            "PYTHON-UTSU-SAS": ("python-utsusemi-SAS", ["python-utsusemi"], "python-utsusemi"),
            "PYTHON-EMAKI": ("python-emaki", ["python-utsusemi"], "python"),
            "MANYO-DNA": ("manyo-DNA", ["manyo-utsusemi"], "manyo"),
            "LIB-LEVMAR": ("lib-levmar", [], "libs"),
            "MANYO-LEVMAR": ("manyo-levmar", ["manyo-core", "lib-levmar"], "manyo"),
            "PYTHON-UTSU-DNA": ("python-utsusemi-DNA", ["manyo-levmar"], "python-utsusemi"),
            "ENVIRON-DNA": ("environ-DNA", ["python-utsusemi-DNA"], "env")
            }


def UtsusemiDefaultInstall(num_of_cores=8):
    print ""
    print "---------------------------------"
    print "  Select Package you install"
    print "---------------------------------"
    print " [ 1] BL01 4SEASONS (SIK)"
    # print " [ 2] BL02 DNA (DNA)"
    print " [14] BL14 AMATERAS (AMR)"
    print " [11] BL11 PLANET (HPN)"
    print " [15] BL15 Taikan (SAS)"
    print " [TK] TKtools"
    print "  "
    print " [ALL] All"
    print " [-1] Exit menu"
    print "---------------------------------"
    print "Which beamline do you want to install ? "
    print "   Input sample : BL01 and BL14 => 1,14"
    ret_BL = raw_input(">> BL = ")
    ret_BL_list = ret_BL.split(",")
    InputBLList = []
    # ENABLE_BL = ["1","2","11","14","15","TK"]
    ENABLE_BL = ["1", "11", "14", "15", "TK"]
    for a_BL in ret_BL_list:
        a_BL = a_BL.strip().upper()
        if a_BL == "ALL":
            InputBLList = ENABLE_BL[:]
        elif a_BL in ENABLE_BL:
            InputBLList.append(a_BL)
        elif a_BL == "-1":
            InputBLList = []

    if len(InputBLList) == 0:
        return
    InputBLList.sort()

    ITU = InstallToUbuntu()
    ITU.installUbuntuPackages()
    ITU.MakeDic["NUM_OF_THREADS"] = num_of_cores
    install_scenario = []
    install_scenario.append("MANYO-CORE")
    install_scenario.append("MANYO-UTSU")
    install_scenario.append("PYTHON-UTSU")
    env_list = []
    for a_BL in InputBLList:
        if a_BL == "1":
            install_scenario.append("PYTHON-UTSU-SIK")
            env_list.append("SIK")
        if a_BL == "2":
            install_scenario.append("MANYO-DNA")
            install_scenario.append("LIB-LEVMAR")
            install_scenario.append("MANYO-LEVMAR")
            install_scenario.append("PYTHON-UTSU-DNA")
            install_scenario.append("ENVIRON-DNA")
            env_list.append("DNA")
        if a_BL == "11":
            install_scenario.append("PYTHON-UTSU-HPN")
            env_list.append("HPN")
        if a_BL == "14":
            install_scenario.append("PYTHON-UTSU-AMR")
            env_list.append("AMR")
        if a_BL == "15":
            install_scenario.append("MANYO-SAS")
            install_scenario.append("PYTHON-UTSU-SAS")
            install_scenario.append("PYTHON-EMAKI")
            env_list.append("SAS")
        if a_BL == "TK":
            install_scenario.append("MANYO-TKT")
            install_scenario.append("PYTHON-UTSU-TKT")

    install_scenario_pack = []
    for a_scenario in install_scenario:
        install_scenario_pack.append(PACK_DIC[a_scenario])

    if ITU.Check_Scenario(install_scenario_pack):
        ret = ITU.Install_as_Scenario(install_scenario_pack)
        if (ret):
            ITU.make_Utsusemi_Env()
            for a_env in env_list:
                ITU.add_EnvironmaneVariables(a_env)
            print "  Utsusemi Default Install was completed."


if __name__ == '__main__':
    if (len(sys.argv) == 2) and (sys.argv[1] == "-all"):
        UtsusemiDefaultInstall()
    else:
        ITU = InstallToUbuntu()
        print "================================================================="
        print "       Welcome to Utsusemi software installer (ver.%s) " % (__version__)
        print "================================================================="
        menu_title = " in common directory "
        continue_flag = True
        while(continue_flag):
            force_return_flag = False
            ITU._showEnviron()
            print """

-----------------------------------------------------------------
       Utsusemi Install Main Menu for Debian or Ubuntu
-----------------------------------------------------------------

  0... Install ALL modules for an instrument
         note: this will DELETE all of modules installed before.

  ----------------------------------------------------
  1... Required Ubuntu Packages
  2... Utsusemi software (Manyo-lib and common code)
  3... Utsusemi software for each BL

  7... Change Install Directories

  9... Uninstall

others.. exit
-----------------------------------------------------------------"""

            ret = raw_input(">> Which ? ")
            if ret not in ["0", "1", "2", "3", "7", "9"]:
                print "Exit without installation."
                continue_flag = False
                force_return_flag = True
            else:
                choice = int(ret)

                if choice == 9:
                    print " "
                    print "Do you want to delete Raw data or xml data, too ?"
                    ret = raw_input(">> answer ?[y/N] ")
                    if ret == "Y" or ret == "y":
                        ret = ITU.uninstall(True)
                    else:
                        ret = ITU.uninstall()
                    if ret == -1:
                        print "Failed to uninstall."
                        sys.exit()
                    print "Uninstall is finished."
                    print " "

                elif choice == 7:
                    while(True):
                        print """  ------------------------------------------------
  Install Directories Menu
  ----------------
  1..... Change Directories to private area
  2..... Return Directories to MLF default
  3..... Load Previous Install Directories
  4..... Show Present Setting

  0..... Return to Main menu
"""
                        ret1 = raw_input(">> Which ? ")
                        if ret1 not in ["0", "1", "2", "3", "4"]:
                            continue
                        if ret1 == "0":
                            force_return_flag = True
                            break
                        elif ret1 == "1":
                            ret = ITU.AskPrivateInstallPath()
                            if ret != 0:
                                print "Something wrong"
                            menu_title = " in private directory "
                        elif ret1 == "2":
                            ITU._initialEnviron()
                        elif ret1 == "3":
                            ITU._readEnviron()
                            ITU._showEnviron()
                        else:
                            ITU._showEnviron()

                elif choice == 1:
                    ITU.installUbuntuPackages()

                elif choice == 2:
                    print "For compiling Manyo-lib, please select the number of threads."
                    ret = raw_input(">> How many threads? [>1, default=1] ")
                    try:
                        nc = int(ret)
                    except:
                        nc = 8
                    ITU.MakeDic["NUM_OF_THREADS"] = nc
                    install_scenario = []
                    install_scenario.append(PACK_DIC["MANYO-CORE"])
                    install_scenario.append(PACK_DIC["MANYO-UTSU"])
                    install_scenario.append(PACK_DIC["PYTHON-UTSU"])
                    install_scenario.append(PACK_DIC["PYTHON-EMAKI"])

                    if ITU.Check_Scenario(install_scenario):
                        ret = ITU.Install_as_Scenario(install_scenario)
                        if (ret):
                            pass
                        else:
                            print " "
                            print "!!!!!!!!!  Install Failed !!!!!!!!!"
                            print "     Utsusemi is NOT installed."
                            print "-----------------------------------"
                            print " Please send install_log.txt to "
                            print "  your contact person."

                elif choice == 3:
                    print " "
                    ret = raw_input(">> Have you installed ManyoLib and Utsusemi Base system ( menu number 2 ) [y/N]?")
                    if ret != "Y" and ret != "y":
                        pass
                    else:
                        print "---------------------------------"
                        print "  Select Package you install"
                        print "---------------------------------"
                        print " [ 1] BL01 4SEASONS (SIK)"
                        # print " [ 2] BL02 DNA (DNA)"
                        print " [14] BL14 AMATERAS (AMR)"
                        print " [11] BL11 PLANET (HPN)"
                        print " [15] BL15 Taikan (SAS)"
                        print " [TK] TKtools"
                        print " [-1] Exit menu"
                        print "---------------------------------"
                        print " "

                        pre_installed_pack = ["manyo-core", "manyo-utsusemi", "python-utsusemi"]
                        while(True):
                            install_scenario = []
                            ret_BL = raw_input(">> Which Package?")
                            ret_BL = ret_BL.strip().upper()
                            BL_name = ""
                            if ret_BL == "1":
                                install_scenario.append(PACK_DIC["PYTHON-UTSU-SIK"])
                                BL_Name = "SIK"
                                break
                            """
                            if ret_BL=="2":
                                install_scenario.append( PACK_DIC["MANYO-DNA"] )
                                install_scenario.append( PACK_DIC["LIB-LEVMAR"] )
                                install_scenario.append( PACK_DIC["MANYO-LEVMAR"] )
                                install_scenario.append( PACK_DIC["PYTHON-UTSU-DNA"] )
                                install_scenario.append( PACK_DIC["ENVIRON-DNA"] )
                                BL_Name="DNA"
                                break
                            """
                            if ret_BL == "14":
                                install_scenario.append(PACK_DIC["PYTHON-UTSU-AMR"])
                                BL_Name = "AMR"
                                break
                            elif ret_BL == "11":
                                install_scenario.append(PACK_DIC["PYTHON-UTSU-HPN"])
                                BL_Name = "HPN"
                                break
                            elif ret_BL == "15":
                                install_scenario.append(PACK_DIC["MANYO-SAS"])
                                install_scenario.append(PACK_DIC["PYTHON-UTSU-SAS"])
                                BL_Name = "SAS"
                                break
                            elif ret_BL == "TK":
                                install_scenario.append(PACK_DIC["MANYO-TKT"])
                                install_scenario.append(PACK_DIC["PYTHON-UTSU-TKT"])
                                BL_Name = "TKtools"
                                break
                            elif ret_BL == "-1":
                                break
                            else:
                                print ' "%s" is Invalid value' % (ret)

                        if len(install_scenario) != 0:
                            if ITU.Check_Scenario(install_scenario, pre_installed_pack):
                                ret = ITU.Install_as_Scenario(install_scenario, pre_installed_pack)
                                if (ret):
                                    ITU.make_Utsusemi_Env()
                                    ITU.add_EnvironmaneVariables(BL_Name)
                                else:
                                    print " "
                                    print "!!!!!!!!!  Install Failed !!!!!!!!!"
                                    print "     Utsusemi is NOT installed."
                                    print "-----------------------------------"
                                    print " Please send install_log.txt to "
                                    print "  your contact person."
                            else:
                                print "Install Scenario is invalid."

                elif choice == 0:
                    print "For compiling Manyo-lib, please select the number of threads."
                    ret = raw_input(">> How many threads? [>1, default=1] ")
                    try:
                        nc = int(ret)
                    except:
                        nc = 8
                    UtsusemiDefaultInstall(nc)

                else:
                    print " Invalid input "

            if force_return_flag:
                pass
            else:
                ret = raw_input(">> Back to Main Menu? [Y/n]")
                if ret == "N" or ret == "n":
                    continue_flag = False
                else:
                    continue_flag = True



#!/usr/bin/python
# -*- coding: utf-8 -*-

from __future__ import print_function
#import time,sys,os,math,glob,datetime,re,urllib,httplib,socket
import time,sys,os,math,glob,datetime,re,urllib,socket
import Manyo as mm
import Manyo.Utsusemi as mu
import Manyo.DNA as md

import utsusemi.ana.Reduction.BaseCommandsInEla as BaseCom
import utsusemi.DNA.ana.Reduction.PeakFit as PF
import shutil
ver = float(sys.version_info[0])+float(sys.version_info[1])*0.1
if ver<2.5:
    from elementtree.ElementTree import ElementTree
else:
    from xml.etree.ElementTree import ElementTree

from utsusemi.DNA.ana.Reduction.DefaultInstParams import *
socket.setdefaulttimeout(1)
#------------------------------------------------------------------------------------------------------------

#------------------------------------------------------------------------------------------------------------
def CheckSampleTempByRunNumber(runNo,temptype): 
        """ 
         Check Sample Temperature by using LakeShore-340 
         @param runNo      (int) 
         @param temptype   (string) 
         @retval EC        (ElementContainer) 
 
        """ 
        TimeParam="-1.0,-1.0" 
        
        from utsusemi.DNA.ana.Reduction.BaseCommandsDNA import CheckRealTime
        startday,starttime,endday,endtime = CheckRealTime(runNo,TimeParam) 
        
        CopyLogFile(startday,endday) 
        import utsusemi.DNA.ana.Reduction.TempMonitor as TM 
        resultEC = TM.CheckSampleTemp(startday,starttime,endday,endtime,temptype,True) 
        resultEC.AddToHeader("RUNNUMBER",str(runNo)) 
        return resultEC
#------------------------------------------------------------------------------------------------------------
def CopyLogFile(startday,endday):

    filename = 'Log_TC_'

    filename1 = filename + startday + ".csv"
    filename2 = filename + endday + ".csv"
    
    orgdatapath = os.path.join(os.environ["UTSUSEMI_DATA_DIR"],"LS340Log")
    #savedatapath = os.path.join(os.environ["HOME"],"ana","tmp")
    savedatapath = os.path.join(mu.UtsusemiEnvGetUserDir(),"ana","tmp")
    
    orgfile1 = os.path.join(orgdatapath,filename1)
    savefile1 = os.path.join(savedatapath,filename1)

    orgfile2 = os.path.join(orgdatapath,filename2)
    savefile2 = os.path.join(savedatapath,filename2)

    cmdheader = "LS340ASMWLogReader >>"
    
    print("%s >> CopyLogFile start" %(cmdheader))
    
    if (not os.path.exists(savefile1) or (os.path.getmtime(savefile1)<os.path.getmtime(orgfile1))):
        print("%s >> Copy %s to %s start" %(cmdheader, orgfile1, savefile1))
        shutil.copy2(orgfile1,savefile1)
    
    else:
        print("%s >> %s is ready for read." %(cmdheader,savefile1))

    if (startday != endday):
        if (not os.path.exists(savefile2) or (os.path.getmtime(savefile2)<os.path.getmtime(orgfile2))):
            print("%s >> Copy %s to %s start" %(cmdheader, orgfile2, savefile2))
            shutil.copy2(orgfile2,savefile2)
        else:
            print("%s >> %s is ready for read." %(cmdheader,savefile1))
    print("%s >> CopyLogFile end" %(cmdheader))

    return
#------------------------------------------------------------------------------------------------------------



from __future__ import print_function
import sys
import urllib
import datetime


class getBeamCurrent(object):

    def __init__(self, startday=None, starttime=None, endday=None, endtime=None):
        if ((starttime is None) and (endtime is None)):
            print(
                "usage:python getBeamCurrent.py YYYY/mm/dd HH:MM:SS YYYY/mm/dd HH:MM:SS")
            return None
        self.s_day = startday
        self.s_time = starttime
        self.e_day = endday
        self.e_time = endtime
        self.dataList = []

        #
        # transform string to datetime.date
        # if fail return today
        #
    def _sDate2date(self, sDate):
        # sDate:'YYYY/mm/dd'
        dat = sDate.split('/')
        if 3 == len(dat):
            return datetime.date(int(dat[0]), int(dat[1]), int(dat[2]))
        else:
            return datetime.date.today()

        #
        # transform string to datetime.datetime
        # if fail return now
        #
    def _sTime2datetime(self, sTime):
        # sTime:'YYYY-mm-dd HH:MM:SS'
        dat = sTime.split(' ')
        date = dat[1].split('-')
        clock = dat[2].split(':')
        if 3 == len(date) and 3 == len(clock):
            return datetime.datetime(int(date[0]), int(date[1]), int(date[2]), int(clock[0]), int(clock[1]), int(clock[2]))
        else:
            return datetime.datetime.now()
        #
        # get one day data list from http://www-cont.j-parc.jp/MLF/cgi/
        #

    def _initializeDataList(self, getDate):
        # getDate:'YYYY/mm/dd'
        urlHead = 'http://www-cont.j-parc.jp/MLF/cgi/'
        url = urlHead + getDate + '/'
        # get file list
        print("url={}".format(url))
        src = urllib.urlopen(url).read()
        if -1 != src.find('404 Not Found'):
            return False
        line = src.split('</a>')
        urlList = []
        # get url list of YYYY-mm-ddTHH_MMBeamCurrent
        for dat in line:
            if(-1 != dat.find('BeamCurrent')):
                urlList.append(url + dat.split(' ')[2])
        src = ''
        # get data list from url list
        for dat in urlList:
            src = src + urllib.urlopen(dat).read()
        for line in src.split('\n'):
            dat = line.split('|')
            # print "len(dat)=",len(dat)
            if 5 == len(dat) and ' year ' != dat[1]:
                self.dataList.append(dat)
        print("loaded len={}".format(len(self.dataList)))
        return True

        #
        # output temperature list from data list
        #
    def _getBCList(self, sBegin, sEnd):
        i = 0
        begin = self._sTime2datetime(' ' + sBegin.replace('/', '-'))
        end = self._sTime2datetime(' ' + sEnd.replace('/', '-'))
        for i in range(0, len(self.dataList)):
            if begin <= self._sTime2datetime(self.dataList[i][0]):
                break
            nBegin = i
            for i in range((len(self.dataList) - 1), -1, -1):
                if end >= self._sTime2datetime(self.dataList[i][0]):
                    break
            nEnd = i
            nShot = 0
            tCurr = 0.
            if nBegin > nEnd:
                print("No Data")
                nShot = -1
                tCurr = -1.
            else:
                # output 'date' 'tag' 'ct9'
                # print "nBegin,nEnd+1,len(dataList)=",nBegin,(nEnd+1),len(self.dataList)
                for i in range(nBegin, (nEnd + 1)):
                    # print self.dataList[i][0]+' '+self.dataList[i][2]+' '+self.dataList[i][4]
                    nShot = nShot + 1
                    tCurr = tCurr + float(self.dataList[i][4])
        # total number of shot and total ct9
                print("Number of shot:{}, Total proton: {} [TP]".format(nShot, tCurr))
            return nShot, tCurr

    def DoIt(self):
        #
        # main routine
        #
        startday = self.s_day
        starttime = self.s_time
        endday = self.e_day
        endtime = self.e_time
        begin = self._sTime2datetime(' ' + startday.replace('/', '-') + ' ' + starttime)
        end = self._sTime2datetime(' ' + endday.replace('/', '-') + ' ' + endtime)
        if begin > end:
            print("Error:'" + startday + ' ' + starttime + "' > '" + endday + " " + endtime)
            return -1, -1.
        d = self._sDate2date(startday)
        dEnd = self._sDate2date(endday)
        td = dEnd - d
        if td.days < 8:
            td = datetime.timedelta(days=1)
        # get data list every day
            while True:
                sDate = str(d.year) + '/' + str(d.month) + '/' + str(d.day)
                if not(self._initializeDataList(sDate)):
                    print("No data in {}".format(sDate))
                    return -1, -1.
                d = d + td
                if d > dEnd:
                    break
            return self._getBCList(startday + ' ' + starttime, endday + ' ' + endtime)
        else:
            print("Overflow of days (max:7days)")



import numpy

import os,time

from vis.UtilPlot import *
import vis.VisContMParams as VCP

import Manyo as mm
import Manyo.Utsusemi as mu
import threading

class D4Mat(object):
    """
    This is python lapping for commands treating D4Matrix on Manyo-lib
    Required Manyo-lib revision is 2250 or later.
    [inamura 160227] : bugfix of wrong path treatment on Windows Environment
    """
    def __init__(self):
        """
        Constructor
        """
        self.SlicedECM = None
        self.D4Mat = mu.UtsusemiD4Matrix()
        self.CommentHead = "D4Mat >>>"
        self.progress = 0
        self.progress_max = 0
        self.AddedFileList = []
        
    def new(self, A1range,A2range,A3range,A4range,titles,paramfilepath):
        """
        makes new matrix

        @param A1range (list) [A1min, A1max, dA1]
        @param A2range (list) [A2min, A2max, dA2]
        @param A3range (list) [A3min, A3max, dA3]
        @param A4range (list) [A4min, A4max, dA4]
        @param titles  (list of string) [ AX1_title, AX2_title, AX3_title, AX4_title ]
        @param datadir (string)
        @param paramfile (string) 
        @return None
        """
        
        if (type(A1range)!=type([])) or (len(A1range)!=3):
            print self.CommentHead + "Invalid parameter about Q1 range. "
            raise
        if (type(A2range)!=type([])) or (len(A2range)!=3):
            print self.CommentHead + "Invalid parameter about Q2 range. "
            raise
        if (type(A3range)!=type([])) or (len(A3range)!=3):
            print self.CommentHead + "Invalid parameter about Q3 range. "
            raise
        if (type(A4range)!=type([])) or (len(A4range)!=3):
            print self.CommentHead + "Invalid parameter about hw range. "
            raise
        if (type(titles)!=type([]) or (len(titles)!=4)):
            print self.CommentHead + "Invalid parameter about titles. "
            raise
        if (type(paramfilepath)!=type("")):
            print self.CommentHead + "Invalid parameter about param file. "
            raise
            
        a1v = mm.ListToDoubleVector( A1range )
        a2v = mm.ListToDoubleVector( A2range )
        a3v = mm.ListToDoubleVector( A3range )
        a4v = mm.ListToDoubleVector( A4range )
        tlv = mm.MakeStringVector()
        for title in titles:
            tlv.append( str(title) )
        ##[inamura 160227]-->  # for Windows Environment
        #datadir = os.path.dirname( paramfilepath ) + "/"
        #paramfile = paramfilepath.replace( datadir, "" )
        datadir = os.path.dirname( paramfilepath )
        paramfile = os.path.basename( paramfilepath )
        ##<--[inamura 160227]
        self.D4Mat.AllocateNewMat( a1v, a2v, a3v, a4v, tlv, datadir, paramfile )
        
        if not self.D4Mat.isStatusOK():
            print self.CommnetHead + "Fail to allocate New matrix.. "
            raise
        
        
    def open(self,paramfilepath):
        """
        Opens existed Matrix data
        
        @param paramfilepath (string) full path to parameter xml file of D4Matrix
        @retval -1  Error happened
        @retval  0  No trouble
        """
        if not os.path.exists(paramfilepath):
            print self.CommentHead + "Cannot find file " + paramfilepath
            return -1
                        
        ##[inamura 160227]-->  # for Windows Environment
        #datadir = os.path.dirname( paramfilepath ) + "/"
        #filename = paramfilepath.replace( datadir, "" )
        datadir = os.path.dirname( paramfilepath )
        filename = os.path.basename( paramfilepath )
        ##<--[inamura 160227]
        print self.CommentHead + "Data dir = ",datadir
        print self.CommentHead + "param_file =",filename
        self.D4Mat.OpenMat( datadir,filename )

        if not self.D4Mat.isStatusOK():
            print self.CommentHead + "Fail to open Matrix.. "
            return -1

        
        return 0


    def close(self):
        """
        Closes opened Matrix data
        
        @param None
        @retval None
        """
        
        self.D4Mat.CloseMat()

    def rebin(self,paramfile1,paramfile2):
        """
        Rebins a matrix to the other matrix

        @param paramfile1    full path to param xml file of original matrix
        @param paramfile2    full path to param xml file of matrix to be rebined
        @retval  -1   Error happened
        @retval   0   No trouble
        """
        if not os.path.exists(paramfile1):
            print self.CommentHead + "Cannot find file " + paramfile1
            return -1
        if not os.path.exists(paramfile2):
            print self.CommentHead + "Cannot find file " + paramfile2
            return -1
                        
        ##[inamura 160227]-->  # for Windows Environment
        #datadir1 = os.path.dirname( paramfile1 ) + "/"
        #filename1 = paramfile1.replace( datadir1, "" )
        #datadir2 = os.path.dirname( paramfile2 ) + "/"
        #filename2 = paramfile2.replace( datadir2, "" )
        datadir1 = os.path.dirname( paramfile1 )
        filename1 = os.path.basename( paramfile1 )
        datadir2 = os.path.dirname( paramfile2 )
        filename2 = os.path.basename( paramfile2 )
        ##<--[inamura 160227]

        print self.CommentHead + "Rebin from:"
        print self.CommentHead + "      Data dir = ",datadir1
        print self.CommentHead + "      param_file =",filename1
        print self.CommentHead + "Rebin to:"
        print self.CommentHead + "      Data dir = ",datadir2
        print self.CommentHead + "      param_file =",filename2

        self.D4Mat.CloseMat()
        self.D4Mat.AddMatToMat( str(datadir1), str(filename1), str(datadir2), str(filename2) )

        if not self.D4Mat.isStatusOK():
            print self.CommentHead + "Fail to rebin Matrix.. "
            raise
        
        return 0
    
    def isOpened(self):
        """
        Returns True if any matrix data has been opened.
        @param None
        @retval True   matrix data has been opened.
        @retval False  not opened.
        """
        return self.D4Mat.isMatOpened()

    def putOpenedPath(self):
        """
        Returns path to parameter xml file of opened D4Matrix

        @param None
        @retval tupple of string ( directory, xml_file_name )
        """
        ret = self.D4Mat.PutOpenedDataPath()
        if ret.size()<2:
            return None

        return ( ret[0], ret[1] )
    
        
    def setTitles(self,titles,isSaved=False):
        """
        Sets other titles

        @param titles ( list of string ) [AX1 title,AX2 title,AX3 title,AX4 title]
        @param isSaved  (bool) If True, rewrite parameter xml file
        @return None
        """
        if ((type(titles)!=type([])) or (len(titles)!=4)):
            print self.CommentHead + " First Argument must be a list of 4 strings. "
            raise
        if (type(isSaved)!=type(True)):
            print self.CommentHead + " Second Argument must be bool. "
            raise
        
        tv = mm.MakeStringVector()
        for title in titles:
            tv.append( str(title) )
        self.D4Mat.SetAxTitles( tv, isSaved )
        
        ##for i,title in enumerate(titles):
        ##    self.D4Mat.SetAxTitle( i, str(title) )

        
    def addFromDirectry(self,filedir):
        """
        Adds data from files by given directory

        @param filedir (string) path to directory stored text file
        @return List of file path to be loaded
        """
        if not os.path.exists( filedir ):
            print self.CommnetHead + "Cannot find such directory, "+filedir
            raise
        
        filelist = os.listdir(filedir)
        filelist.sort()
        
        num_files = len(filelist)

        self.AddedFileList = []
        self.progress = 0
        self.progress_max = num_files
        print "self.progress_max=",self.progress_max
        for filename in filelist:
            filepath =  os.path.join( filedir, filename )
            print self.CommentHead+"filepath = "+filepath+( " :%d/%d" % (self.progress+1, num_files) ) ##[inamura 170116]
            if filepath!=None:
                time.sleep(0.2)
                self.AddedFileList.append( self.addFromFile(filepath) )
            self.progress +=1
        
        self.progress +=1
        
        ##[inamura 170116]-->
        print self.CommentHead+"--------------------------------------------------"
        print self.CommentHead+" Finished : Load vbin files"
        print self.CommentHead+" in %s"%(filedir)
        print self.CommentHead+"--------------------------------------------------"
        ##<--[inamura 170116]
        return self.AddedFileList

    def addFromFile(self,filename):
        """
        Adds data from a file

        @param filename (string) path to data file
        @return file path to be loaded
        """
        st = time.time()
        try:
            if filename[-5:]==".vbin":
                self.D4Mat.AddToMatFromBin( str( filename ) )
            elif filename[-4:]==".txt":
                self.D4Mat.AddToMatFromText( str( filename ) )
            else:
                print self.CommentHead + " File name is invalid. "
                raise
            
            if not self.D4Mat.isStatusOK():
                print self.CommnetHead + "Fail to add data from file.. "
                raise

        except:
            print self.CommentHead+"Cannot read file :",filename
            filename = None
        print self.CommentHead+"time of cost = ",time.time()-st
        return filename
            

    def subtractText( self,filename ):
        """
        Subtracts data of text file from Matrix

        @param filename (string) path to data of text file
        @retval -1  Error happened
        @retval  0  No trouble
        """

        try:
            self.D4Mat.SubtractFromMatByText( str( filename ) )
            if not self.D4Mat.isStatusOK():
                print self.CommnetHead + "Fail to subtract text file data from D4Mat.. "
                return -1
        except:
            print self.CommentHead + "Fail to open text file.."
            return -1
        
        return 0
            

    def sliceMat(self, A1range, A2range, A3range, A4range, type_axes, foldings ):
        """
        Slices Matrix

        @param A1range    (list of float) range of AX1
        @param A2range    (list of float) range of AX2
        @param A3range    (list of float) range of AX3
        @param A4range    (list of float) range of AX4
        @param type_axes  (list of string) type of axes 'X','Y' or 'T'
        @param foldings   (list of double )
        @return None
        """
        
        if (type(A1range)!=type([])) or (len(A1range)!=2):
            print self.CommnetHead + "Invalid parameter about Q1 range. "
            raise
        if (type(A2range)!=type([])) or (len(A2range)!=2):
            print self.CommnetHead + "Invalid parameter about Q2 range. "
            raise
        if (type(A3range)!=type([])) or (len(A3range)!=2):
            print self.CommnetHead + "Invalid parameter about Q3 range. "
            raise
        if (type(A4range)!=type([])) or (len(A4range)!=2):
            print self.CommnetHead + "Invalid parameter about hw range. "
            raise
        if (type(type_axes)!=type([])) or (len(type_axes)!=4):
            print self.CommnetHead + "Invalid type_axes. "
            raise
        
        a1v = mm.ListToDoubleVector( A1range )
        a2v = mm.ListToDoubleVector( A2range )
        a3v = mm.ListToDoubleVector( A3range )
        a4v = mm.ListToDoubleVector( A4range )
        
        
        cnt_Ax = {'X':0,'Y':0,'T':0}
        def_axes = mm.MakeStringVector()
        isValid = True
        for Axtype in type_axes:
            if Axtype=='X':
                cnt_Ax['X'] += 1
            elif Axtype=='Y':
                cnt_Ax['Y'] += 1
            elif Axtype=='T':
                cnt_Ax['T'] += 1
            else:
                break;
            def_axes.append(Axtype)

        if (cnt_Ax['X']!=1) or (cnt_Ax['Y']!=1) or (cnt_Ax['T']!=2):
            print self.CommentHead + "Invalid argument type_axes."
            isValid = False
            
        fld = mm.ListToDoubleVector( foldings ) ##[inamura 160616]
        if isValid:
            self.D4Mat.SliceMat( a1v, a2v, a3v, a4v, def_axes, fld )
            if not self.D4Mat.isStatusOK():
                print self.CommentHead + "Fail to slice data.. "
                raise
            self.SlicedECM = self.D4Mat.PutSlicedECA()
        else:
            raise

    #[inamura 170227]-->
    def sliceMat3d(self, A1range, A2range, A3range, A4range, type_axes, foldings, name_axes ):
        """
        Slices Matrix 3D

        @param A1range    (list of float) range of AX1
        @param A2range    (list of float) range of AX2
        @param A3range    (list of float) range of AX3
        @param A4range    (list of float) range of AX4
        @param type_axes  (list of string) type of axes 'X','Y','Z' or 'T'
        @param foldings   (list of int )
        @return None
        """
        
        if (type(A1range)!=type([])) or (len(A1range)!=2):
            print self.CommnetHead + "Invalid parameter about Q1 range. "
            raise
        if (type(A2range)!=type([])) or (len(A2range)!=2):
            print self.CommnetHead + "Invalid parameter about Q2 range. "
            raise
        if (type(A3range)!=type([])) or (len(A3range)!=2):
            print self.CommnetHead + "Invalid parameter about Q3 range. "
            raise
        if (type(A4range)!=type([])) or (len(A4range)!=2):
            print self.CommnetHead + "Invalid parameter about hw range. "
            raise
        if (type(type_axes)!=type([])) or (len(type_axes)!=4):
            print self.CommnetHead + "Invalid type_axes. "
            raise
        
        a1v = mm.ListToDoubleVector( A1range )
        a2v = mm.ListToDoubleVector( A2range )
        a3v = mm.ListToDoubleVector( A3range )
        a4v = mm.ListToDoubleVector( A4range )
        
        cnt_Ax = {'X':0,'Y':0,'Z':0,'T':0}
        def_axes = mm.MakeStringVector()
        isValid = True
        for Axtype in type_axes:
            if Axtype=='X':
                cnt_Ax['X'] += 1
            elif Axtype=='Y':
                cnt_Ax['Y'] += 1
            elif Axtype=='Z':
                cnt_Ax['Z'] += 1
            elif Axtype=='T':
                cnt_Ax['T'] += 1
            else:
                break;
            def_axes.append(Axtype)

        if (cnt_Ax['X']!=1) or (cnt_Ax['Y']!=1) or (cnt_Ax['Y']!=1) or (cnt_Ax['T']!=1):
            print self.CommentHead + "Invalid argument type_axes."
            isValid = False
            
        fld = mm.ListToDoubleVector( foldings )
        keys = mm.MakeStringVector(3)
        for i in range(3):
            keys[i] = str( name_axes[i] )
        
        if isValid:
            try:
                del self.SlicedECM
            except:
                pass
            self.SlicedECM = mm.ElementContainerMatrix()
            self.D4Mat.SliceMat3D( self.SlicedECM, a1v, a2v, a3v, a4v, def_axes, fld, keys )
            if not self.D4Mat.isStatusOK():
                print self.CommnetHead + "Fail to slice data.. "
                raise
        else:
            raise
        #<--[inamura 170227]

    def loadVisContParams(self,filename=""):
        """
        Pick up information of Axis from VisCont parameter file

        @param filename   path to XML parameter file produced by VisualCont2
        @return  tupple of list ( list_of_ranges_for_axes, list_of_titles_for_axes )
        """
        if not os.path.exists(filename):
            ss = self.CommentHead+(" Cannot find XML file(%s)" % (filename))
            raise "FileIOError",ss

        vcp = VCP.VisContMParams(filename)

        axrange_list = []
        title_list = []
        for id in range(4):
            (type_str, r_axis, folding) = vcp.paxis_info[str(id)]
            ##[inamura 160622]-->
            """
            if type_str in ["x","y"]:
                r_vec = self.D4Mat.CalcRangeAsBinCenterZero( r_axis[0], r_axis[1], r_axis[2] )
                r_axis[0] = r_vec[0]
                r_axis[1] = r_vec[1]
            """
            if type_str in ["x","y"]:
                pass
            else:
                r_step = r_axis[1] - r_axis[0]
                if len(r_axis)==2:
                    r_axis.append( r_step )
                elif len(r_axis)==3:
                    r_axis[2] = r_step
                else:
                    raise
            ##<--[inamura 160622]
            axrange_list.append(r_axis)
            title_list.append( vcp.proj_axes[str(id)][4] )
            
        return (axrange_list,title_list)

    def makeNewMatFromVisContParams(self, VisContParamFile="", D4MatParamFile=""):
        """
        Makes new D4Matrix data file using parameters fo VisContParam

        @param VisContParamFile   path to XML parameter file produced by VisualCont2
        @param D4MatParamFile     path to XML parameter file for D4Matrix
        @return None
        """
        (range_list,title_list) = self.loadVisContParams( VisContParamFile )

        self.new(range_list[0],range_list[1],range_list[2],range_list[3],title_list,D4MatParamFile )

